@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005,
+   2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -39,7 +40,7 @@
 #endif  
 
 #include <stdlib.h>
-#include <mailutils/libsieve.h>
+#include <mailutils/sieve.h>
 
 struct val_ctr {  /* Data passed to the counter function */
   mu_header_t hdr;   /* Headers of the current message */
@@ -121,6 +122,7 @@ numaddr_test (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
      to vc.limit. If `:under' tag was given we reverse the return value */
   if (mu_sieve_tag_lookup (tags, "under", NULL))
     rc = !rc;
+
   return rc;
 }
 
