@@ -2,9 +2,11 @@
    GNU Mailutils pop3 functions.  This application interactively allows users
    to contact a pop3 server.
 
+   Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009 Free Software Foundation
+
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -14,13 +16,15 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
+# include <config.h>  
+#endif 
 #include <sys/types.h>
+#include <netinet/in.h>
+#include <netdb.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
@@ -29,7 +33,6 @@
 #include <stdlib.h>
 #include <termios.h>
 #include <signal.h>
-#include <ctype.h>
 
 #ifdef WITH_READLINE
 # include <readline/readline.h>
@@ -40,6 +43,10 @@
 #include <mailutils/iterator.h>
 #include <mailutils/error.h>
 #include <mailutils/errno.h>
+#include <mailutils/vartab.h>
+#include <mailutils/argcv.h>
+#include <mailutils/cctype.h>
+#include <mailutils/cstr.h>
 
 /* A structure which contains information on the commands this program
    can understand. */
@@ -71,9 +78,9 @@ int com_top (char *);
 int com_uidl (char *);
 int com_user (char *);
 int com_verbose (char *);
+int com_prompt (char *);
 
 void initialize_readline (void);
-char *stripwhite (char *);
 COMMAND *find_command (char *);
 char *dupstr (const char *);
 int execute_line (char *);
@@ -82,31 +89,29 @@ int valid_argument (const char *, char *);
 void sig_int (int);
 
 COMMAND commands[] = {
-  {"apop", com_apop, "Authenticate with APOP: APOP user secret"},
-  {"capa", com_capa, "List capabilities: capa"},
-  {"disconnect", com_disconnect, "Close connection: disconnect"},
-  {"dele", com_dele, "Mark message: DELE msgno"},
-  {"exit", com_exit, "exit program"},
-  {"help", com_help, "Display this text"},
-  {"?", com_help, "Synonym for `help'"},
-  {"list", com_list, "List messages: LIST [msgno]"},
-  {"noop", com_noop, "Send no operation: NOOP"},
-  {"pass", com_pass, "Send passwd: PASS [passwd]"},
-  {"connect", com_connect, "Open connection: connect hostname [port]"},
-  {"quit", com_quit, "Go to Update state : QUIT"},
-  {"retr", com_retr, "Dowload message: RETR msgno"},
-  {"rset", com_rset, "Unmark all messages: RSET"},
-  {"stat", com_stat, "Get the size and count of mailbox : STAT [msgno]"},
-  {"top", com_top, "Get the header of message: TOP msgno [lines]"},
-  {"uidl", com_uidl, "Get the unique id of message: UIDL [msgno]"},
-  {"user", com_user, "send login: USER user"},
-  {"verbose", com_verbose, "Enable Protocol tracing: verbose [on|off]"},
-  {NULL, NULL, NULL}
+  { "apop", com_apop, "Authenticate with APOP: APOP user secret" },
+  { "capa", com_capa, "List capabilities: capa" },
+  { "disconnect", com_disconnect, "Close connection: disconnect" },
+  { "dele", com_dele, "Mark message: DELE msgno" },
+  { "exit", com_exit, "exit program" },
+  { "help", com_help, "Display this text" },
+  { "?", com_help, "Synonym for `help'" },
+  { "list", com_list, "List messages: LIST [msgno]" },
+  { "noop", com_noop, "Send no operation: NOOP" },
+  { "pass", com_pass, "Send passwd: PASS [passwd]" },
+  { "prompt", com_prompt, "Set command prompt" },
+  { "connect", com_connect, "Open connection: connect hostname [port]" },
+  { "quit", com_quit, "Go to Update state : QUIT" },
+  { "retr", com_retr, "Dowload message: RETR msgno" },
+  { "rset", com_rset, "Unmark all messages: RSET" },
+  { "stat", com_stat, "Get the size and count of mailbox : STAT [msgno]" },
+  { "top", com_top, "Get the header of message: TOP msgno [lines]" },
+  { "uidl", com_uidl, "Get the unique id of message: UIDL [msgno]" },
+  { "user", com_user, "send login: USER user" },
+  { "verbose", com_verbose, "Enable Protocol tracing: verbose [on|off]" },
+  { NULL, NULL, NULL }
 };
 
-/* The name of this program, as taken from argv[0]. */
-char *progname;
-
 /* Global handle for pop3.  */
 mu_pop3_t pop3;
 
@@ -116,6 +121,70 @@ int verbose;
 /* When non-zero, this global means the user is done using this program. */
 int done;
 
+enum pop_session_status
+  {
+    pop_session_disconnected,
+    pop_session_connected,
+    pop_session_logged_in
+  };
+
+enum pop_session_status pop_session_status;
+
+int connect_argc;
+char **connect_argv;
+
+/* Host we are connected to. */
+#define host connect_argv[0]
+int port = 110;
+char *username;
+
+/* Command line prompt */
+#define DEFAULT_PROMPT "pop3> "
+char *prompt;
+
+const char *
+pop_session_str (enum pop_session_status stat)
+{
+  switch (stat)
+    {
+    case pop_session_disconnected:
+      return "disconnected";
+      
+    case pop_session_connected:
+      return "connected";
+      
+    case pop_session_logged_in:
+      return "logged in";
+    }
+  return "unknown";
+}
+
+char *
+expand_prompt ()
+{
+  mu_vartab_t vtab;
+  char *str;
+  
+  if (mu_vartab_create (&vtab))
+    return strdup (prompt);
+  mu_vartab_define (vtab, "user",
+		    (pop_session_status == pop_session_logged_in) ?
+		      username : "not logged in", 1);
+  mu_vartab_define (vtab, "host",
+		    (pop_session_status != pop_session_disconnected) ?
+		      host : "not connected", 1);
+  mu_vartab_define (vtab, "program-name", mu_program_name, 1);
+  mu_vartab_define (vtab, "canonical-program-name", "pop3client", 1);
+  mu_vartab_define (vtab, "package", PACKAGE, 1);
+  mu_vartab_define (vtab, "version", PACKAGE_VERSION, 1);
+  mu_vartab_define (vtab, "status", pop_session_str (pop_session_status), 1);
+  
+  if (mu_vartab_expand (vtab, prompt, &str))
+    str = strdup (prompt);
+  mu_vartab_destroy (&vtab);
+  return str;
+}
+
 char *
 dupstr (const char *s)
 {
@@ -124,7 +193,7 @@ dupstr (const char *s)
   r = malloc (strlen (s) + 1);
   if (!r)
     {
-      fprintf (stderr, "Memory exhausted\n");
+      mu_error ("Memory exhausted");
       exit (1);
     }
   strcpy (r, s);
@@ -157,11 +226,10 @@ initialize_readline ()
    in case we want to do some simple parsing.  Return the array of matches,
    or NULL if there aren't any. */
 char **
-pop_completion (char *text, int start, int end)
+pop_completion (char *text, int start, int end MU_ARG_UNUSED)
 {
   char **matches;
 
-  (void) end;
   matches = (char **) NULL;
 
   /* If this word is at the start of the line, then it is a command
@@ -201,7 +269,7 @@ command_generator (const char *text, int state)
     }
 
   /* If no names matched, then return NULL. */
-  return ((char *) NULL);
+  return NULL;
 }
 
 #else
@@ -227,38 +295,35 @@ readline (char *prompt)
 }
 
 void
-add_history (const char *s ARG_UNUSED)
+add_history (const char *s MU_ARG_UNUSED)
 {
 }
 #endif
 
 
 int
-main (int argc ARG_UNUSED, char **argv)
+main (int argc MU_ARG_UNUSED, char **argv)
 {
   char *line, *s;
 
-  progname = strrchr (argv[0], '/');
-  if (progname)
-    progname++;
-  else
-    progname = argv[0];
-
+  mu_set_program_name (argv[0]);
+  prompt = strdup (DEFAULT_PROMPT);
   initialize_readline ();	/* Bind our completer. */
 
   /* Loop reading and executing lines until the user quits. */
   while (!done)
     {
-
-      line = readline ((char *) "pop3> ");
-
+      char *p = expand_prompt ();
+      line = readline (p);
+      free (p);
+      
       if (!line)
 	break;
 
       /* Remove leading and trailing whitespace from the line.
          Then, if there is anything left, add it to the history list
          and execute it. */
-      s = stripwhite (line);
+      s = mu_str_stripws (line);
 
       if (*s)
 	{
@@ -266,7 +331,7 @@ main (int argc ARG_UNUSED, char **argv)
 	  add_history (s);
 	  status = execute_line (s);
 	  if (status != 0)
-	    fprintf (stderr, "Error: %s\n", mu_strerror (status));
+	    mu_error ("Error: %s", mu_strerror (status));
 	}
 
       free (line);
@@ -278,45 +343,34 @@ main (int argc ARG_UNUSED, char **argv)
 int
 execute_line (char *line)
 {
-  register int i;
   COMMAND *command;
-  char *word;
+  char *word, *arg;
 
   /* Isolate the command word. */
-  i = 0;
-  while (line[i] && isspace (line[i]))
-    i++;
-  word = line + i;
-
-  while (line[i] && !isspace (line[i]))
-    i++;
-
-  if (line[i])
-    line[i++] = '\0';
-
+  word = mu_str_skip_class (line, MU_CTYPE_SPACE);
+  arg = mu_str_skip_class_comp (word, MU_CTYPE_SPACE);
+  if (*arg)
+    {
+      *arg++ = 0;
+      arg = mu_str_skip_class (arg, MU_CTYPE_SPACE);
+    }
+      
   command = find_command (word);
 
   if (!command)
     {
-      fprintf (stderr, "%s: No such command for %s.\n", word, progname);
-      return (-1);
+      mu_error ("%s: No such command.", word);
+      return 0;
     }
 
-  /* Get argument to command, if any. */
-  while (isspace (line[i]))
-    i++;
-
-  word = line + i;
-
   /* Call the function. */
-  return ((*(command->func)) (word));
+  return ((*(command->func)) (arg));
 }
 
 /* Look up NAME as the name of a command, and return a pointer to that
    command.  Return a NULL pointer if NAME isn't a command name. */
 COMMAND *
-find_command (name)
-     char *name;
+find_command (char *name)
 {
   register int i;
 
@@ -327,27 +381,6 @@ find_command (name)
   return ((COMMAND *) NULL);
 }
 
-/* Strip whitespace from the start and end of STRING.  Return a pointer
-   into STRING. */
-char *
-stripwhite (char *string)
-{
-  register char *s, *t;
-
-  for (s = string; isspace (*s); s++)
-    ;
-
-  if (*s == 0)
-    return (s);
-
-  t = s + strlen (s) - 1;
-  while (t > s && isspace (*t))
-    t--;
-  *++t = '\0';
-
-  return s;
-}
-
 int
 com_verbose (char *arg)
 {
@@ -362,7 +395,7 @@ com_verbose (char *arg)
 	{
 	  mu_debug_t debug;
 	  mu_debug_create (&debug, NULL);
-	  mu_debug_set_level (debug, MU_DEBUG_PROT);
+	  mu_debug_set_level (debug, MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
 	  status = mu_pop3_set_debug (pop3, debug);
 	}
       else
@@ -376,14 +409,20 @@ com_verbose (char *arg)
 int
 com_user (char *arg)
 {
+  int status;
+  
   if (!valid_argument ("user", arg))
     return EINVAL;
-  return mu_pop3_user (pop3, arg);
+  status = mu_pop3_user (pop3, arg);
+  if (status == 0)
+    username = strdup (arg);
+  return status;
 }
 
 int
 com_apop (char *arg)
 {
+  int status;
   char *user, *digest;
 
   if (!valid_argument ("apop", arg))
@@ -392,28 +431,31 @@ com_apop (char *arg)
   digest = strtok (NULL, " ");
   if (!valid_argument ("apop", user) || !valid_argument ("apop", digest))
     return EINVAL;
-  return mu_pop3_apop (pop3, user, digest);
+  status = mu_pop3_apop (pop3, user, digest);
+  if (status == 0)
+    {
+      username = strdup (user);
+      pop_session_status = pop_session_logged_in;
+    }
+  return status;
 }
 
 int
-com_capa (char *arg ARG_UNUSED)
+com_capa (char *arg MU_ARG_UNUSED)
 {
-  list_t list = NULL;
-  int status = mu_pop3_capa (pop3, &list);
+  mu_iterator_t iterator = NULL;
+  int status = mu_pop3_capa (pop3, &iterator);
 
   if (status == 0)
     {
-      iterator_t iterator = NULL;
-      iterator_create (&iterator, list);
-      for (iterator_first (iterator);
-	   !iterator_is_done (iterator); iterator_next (iterator))
+      for (mu_iterator_first (iterator);
+	   !mu_iterator_is_done (iterator); mu_iterator_next (iterator))
 	{
 	  char *capa = NULL;
-	  iterator_current (iterator, (void **) &capa);
+	  mu_iterator_current (iterator, (void **) &capa);
 	  printf ("Capa: %s\n", (capa) ? capa : "");
 	}
-      iterator_destroy (&iterator);
-      list_destroy (&list);
+      mu_iterator_destroy (&iterator);
     }
   return status;
 }
@@ -424,22 +466,19 @@ com_uidl (char *arg)
   int status = 0;
   if (arg == NULL || *arg == '\0')
     {
-      list_t list = NULL;
-      status = mu_pop3_uidl_all (pop3, &list);
+      mu_iterator_t uidl_iterator = NULL;
+      status = mu_pop3_uidl_all (pop3, &uidl_iterator);
       if (status == 0)
 	{
-	  iterator_t uidl_iterator = NULL;
-	  iterator_create (&uidl_iterator, list);
-	  for (iterator_first (uidl_iterator);
-	       !iterator_is_done (uidl_iterator);
-	       iterator_next (uidl_iterator))
+	  for (mu_iterator_first (uidl_iterator);
+	       !mu_iterator_is_done (uidl_iterator);
+	       mu_iterator_next (uidl_iterator))
 	    {
 	      char *uidl = NULL;
-	      iterator_current (uidl_iterator, (void **) &uidl);
+	      mu_iterator_current (uidl_iterator, (void **) &uidl);
 	      printf ("UIDL: %s\n", (uidl) ? uidl : "");
 	    }
-	  iterator_destroy (&uidl_iterator);
-	  list_destroy (&list);
+	  mu_iterator_destroy (&uidl_iterator);
 	}
     }
   else
@@ -460,22 +499,19 @@ com_list (char *arg)
   int status = 0;
   if (arg == NULL || *arg == '\0')
     {
-      list_t list = NULL;
-      status = mu_pop3_list_all (pop3, &list);
+      mu_iterator_t list_iterator;
+      status = mu_pop3_list_all (pop3, &list_iterator);
       if (status == 0)
 	{
-	  iterator_t list_iterator;
-	  iterator_create (&list_iterator, list);
-	  for (iterator_first (list_iterator);
-	       !iterator_is_done (list_iterator);
-	       iterator_next (list_iterator))
+	  for (mu_iterator_first (list_iterator);
+	       !mu_iterator_is_done (list_iterator);
+	       mu_iterator_next (list_iterator))
 	    {
 	      char *list = NULL;
-	      iterator_current (list_iterator, (void **) &list);
+	      mu_iterator_current (list_iterator, (void **) &list);
 	      printf ("LIST: %s\n", (list) ? list : "");
 	    }
-	  iterator_destroy (&list_iterator);
-	  list_destroy (&list);
+	  mu_iterator_destroy (&list_iterator);
 	}
     }
   else
@@ -490,7 +526,7 @@ com_list (char *arg)
 }
 
 int
-com_noop (char *arg ARG_UNUSED)
+com_noop (char *arg MU_ARG_UNUSED)
 {
   return mu_pop3_noop (pop3);
 }
@@ -511,10 +547,33 @@ echo_on (struct termios *stored_settings)
   tcsetattr (0, TCSANOW, stored_settings);
 }
 
+int
+com_prompt (char *arg)
+{
+  int quote;
+  size_t size;
+  
+  if (!valid_argument ("prompt", arg))
+    return EINVAL;
+
+  free (prompt);
+  size = mu_argcv_quoted_length (arg, &quote);
+  prompt = malloc (size + 1);
+  if (!prompt)
+    {
+      mu_error ("Memory exhausted");
+      exit (1);
+    }
+  mu_argcv_unquote_copy (prompt, arg, size);
+  return 0;
+}
+
 int
 com_pass (char *arg)
 {
+  int status;
   char pass[256];
+  
   if (!arg || *arg == '\0')
     {
       struct termios stored_settings;
@@ -529,18 +588,21 @@ com_pass (char *arg)
       pass[strlen (pass) - 1] = '\0';	/* nuke the trailing line.  */
       arg = pass;
     }
-  return mu_pop3_pass (pop3, arg);
+  status = mu_pop3_pass (pop3, arg);
+  if (status == 0)
+    pop_session_status = pop_session_logged_in;
+  return status;
 }
 
 int
-com_stat (char *arg ARG_UNUSED)
+com_stat (char *arg MU_ARG_UNUSED)
 {
   unsigned count, size;
   int status = 0;
 
   count = size = 0;
   status = mu_pop3_stat (pop3, &count, &size);
-  fprintf (stdout, "Mesgs: %d Size %d\n", count, size);
+  printf ("Mesgs: %d Size %d\n", count, size);
   return status;
 }
 
@@ -595,7 +657,7 @@ com_help (char *arg)
 }
 
 int
-com_rset (char *arg ARG_UNUSED)
+com_rset (char *arg MU_ARG_UNUSED)
 {
   return mu_pop3_rset (pop3);
 }
@@ -603,7 +665,7 @@ com_rset (char *arg ARG_UNUSED)
 int
 com_top (char *arg)
 {
-  stream_t stream;
+  mu_stream_t stream;
   unsigned int msgno;
   unsigned int lines;
   char *space;
@@ -628,9 +690,9 @@ com_top (char *arg)
     {
       size_t n = 0;
       char buf[128];
-      while ((stream_readline (stream, buf, sizeof buf, 0, &n) == 0) && n)
+      while ((mu_stream_readline (stream, buf, sizeof buf, 0, &n) == 0) && n)
 	printf ("%s", buf);
-      stream_destroy (&stream, NULL);
+      mu_stream_destroy (&stream, NULL);
     }
   return status;
 }
@@ -638,7 +700,7 @@ com_top (char *arg)
 int
 com_retr (char *arg)
 {
-  stream_t stream;
+  mu_stream_t stream;
   unsigned int msgno;
   int status;
 
@@ -652,37 +714,92 @@ com_retr (char *arg)
     {
       size_t n = 0;
       char buf[128];
-      while ((stream_readline (stream, buf, sizeof buf, 0, &n) == 0) && n)
+      while ((mu_stream_readline (stream, buf, sizeof buf, 0, &n) == 0) && n)
 	printf ("%s", buf);
-      stream_destroy (&stream, NULL);
+      mu_stream_destroy (&stream, NULL);
     }
   return status;
 }
 
+int
+get_port (const char *port_str, int *pn)
+{
+  short port_num;
+  long num;
+  char *p;
+  
+  num = port_num = strtol (port_str, &p, 0);
+  if (*p == 0)
+    {
+      if (num != port_num)
+	{
+	  mu_error ("bad port number: %s", port_str);
+	  return 1;
+	}
+    }
+  else
+    {
+      struct servent *sp = getservbyname (port_str, "tcp");
+      if (!sp)
+	{
+	  mu_error ("unknown port name");
+	  return 1;
+	}
+      port_num = ntohs (sp->s_port);
+    }
+  *pn = port_num;
+  return 0;
+}
+
 int
 com_connect (char *arg)
 {
-  char host[256];
-  int port = 110;
   int status;
+  int n = 110;
+  int argc;
+  char **argv;
+  
   if (!valid_argument ("connect", arg))
     return 1;
-  *host = '\0';
-  sscanf (arg, "%256s %d", host, &port);
-  if (!valid_argument ("connect", host))
-    return EINVAL;
-  if (pop3)
+  
+  if (mu_argcv_get (arg, NULL, NULL, &argc, &argv))
+    {
+      mu_error ("Cannot parse arguments");
+      return 0;
+    }
+  
+  if (!valid_argument ("connect", argv[0]))
+    {
+      mu_argcv_free (argc, argv);
+      return EINVAL;
+    }
+  
+  if (argc > 2)
+    {
+      mu_error ("Too many arguments");
+      mu_argcv_free (argc, argv);
+      return 0;
+    }
+
+  if (argc == 2 && get_port (argv[1], &n))
+    {
+      mu_argcv_free (argc, argv);
+      return 0;
+    }
+  
+  if (pop_session_status != pop_session_disconnected)
     com_disconnect (NULL);
+  
   status = mu_pop3_create (&pop3);
   if (status == 0)
     {
-      stream_t tcp;
+      mu_stream_t tcp;
 
       if (verbose)
-	com_verbose ("verbose on");
+	com_verbose ("on");
       status =
-	tcp_stream_create (&tcp, host, port,
-			   MU_STREAM_READ | MU_STREAM_NO_CHECK);
+	mu_tcp_stream_create (&tcp, argv[0], n,
+			      MU_STREAM_READ | MU_STREAM_NO_CHECK);
       if (status == 0)
 	{
 	  mu_pop3_set_carrier (pop3, tcp);
@@ -695,45 +812,61 @@ com_connect (char *arg)
 	}
     }
 
-  if (status != 0)
-    fprintf (stderr, "Failed to create pop3: %s\n", mu_strerror (status));
+  if (status)
+    {
+      mu_error ("Failed to create pop3: %s", mu_strerror (status));
+      mu_argcv_free (argc, argv);
+    }
+  else
+    {
+      connect_argc = argc;
+      connect_argv = argv;
+      port = n;
+      pop_session_status = pop_session_connected;
+    }
+  
   return status;
 }
 
 int
-com_disconnect (char *arg)
+com_disconnect (char *arg MU_ARG_UNUSED)
 {
-  (void) arg;
   if (pop3)
     {
       mu_pop3_disconnect (pop3);
       mu_pop3_destroy (&pop3);
       pop3 = NULL;
+      
+      mu_argcv_free (connect_argc, connect_argv);
+      connect_argc = 0;
+      connect_argv = NULL;
+      pop_session_status = pop_session_disconnected;
     }
   return 0;
 }
 
 int
-com_quit (char *arg ARG_UNUSED)
+com_quit (char *arg MU_ARG_UNUSED)
 {
+  int status = 0;
   if (pop3)
     {
       if (mu_pop3_quit (pop3) == 0)
 	{
-	  mu_pop3_disconnect (pop3);
+	  status = com_disconnect (arg);
 	}
       else
 	{
-	  fprintf (stdout, "Try 'exit' to leave %s\n", progname);
+	  printf ("Try 'exit' to leave %s\n", mu_program_name);
 	}
     }
   else
-    fprintf (stdout, "Try 'exit' to leave %s\n", progname);
-  return 0;
+    printf ("Try 'exit' to leave %s\n", mu_program_name);
+  return status;
 }
 
 int
-com_exit (char *arg ARG_UNUSED)
+com_exit (char *arg MU_ARG_UNUSED)
 {
   if (pop3)
     {
@@ -751,7 +884,7 @@ valid_argument (const char *caller, char *arg)
 {
   if (!arg || !*arg)
     {
-      fprintf (stderr, "%s: Argument required.\n", caller);
+      mu_error ("%s: Argument required", caller);
       return 0;
     }
 
