@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2007 Free Software Foundation, Inc.
+   2007, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -171,7 +171,7 @@ auth_gssapi (struct imap4d_command *command,
   for (;;)
     {
       imap4d_getline (&token_str, &token_size, &token_len);
-      util_base64_decode (token_str, token_len, &tmp, &size);
+      mu_base64_decode (token_str, token_len, &tmp, &size);
       tokbuf.value = tmp;
       tokbuf.length = size;
 
@@ -189,7 +189,7 @@ auth_gssapi (struct imap4d_command *command,
 	{
 	  if (outbuf.length)
 	    {
-	      util_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
+	      mu_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
 	      util_send ("+ %*.*s\r\n", size, size, tmp);
 	      free (tmp);
 	      gss_release_buffer (&min_stat, &outbuf);
@@ -209,7 +209,7 @@ auth_gssapi (struct imap4d_command *command,
 
   if (outbuf.length)
     {
-      util_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
+      mu_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
       util_send ("+ %*.*s\r\n", size, size, tmp);
       free (tmp);
       gss_release_buffer (&min_stat, &outbuf);
@@ -229,13 +229,13 @@ auth_gssapi (struct imap4d_command *command,
       return RESP_NO;
     }
   
-  util_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
+  mu_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
   util_send ("+ %*.*s\r\n", size, size, tmp);
   free (tmp);
 
   imap4d_getline (&token_str, &token_size, &token_len);
-  util_base64_decode (token_str, token_len,
-		      (unsigned char **) &tokbuf.value, &tokbuf.length);
+  mu_base64_decode (token_str, token_len,
+		    (unsigned char **) &tokbuf.value, &tokbuf.length);
   free (token_str);
 
   maj_stat = gss_unwrap (&min_stat, context, &tokbuf, &outbuf, &cflags,
@@ -254,7 +254,7 @@ auth_gssapi (struct imap4d_command *command,
   if ((mech & SUPPORTED_P_MECH) == 0)
     {
       mu_diag_output (MU_DIAG_NOTICE,
-	      _("Client requested unsupported protection mechanism (%d)"),
+	      _("client requested unsupported protection mechanism (%d)"),
 	      mech);
       gss_release_buffer (&min_stat, &outbuf);
       maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
@@ -267,7 +267,7 @@ auth_gssapi (struct imap4d_command *command,
   *username = malloc (outbuf.length - 4 + 1);
   if (!*username)
     {
-      mu_diag_output (MU_DIAG_NOTICE, _("Not enough memory"));
+      mu_diag_output (MU_DIAG_NOTICE, _("not enough memory"));
       gss_release_buffer (&min_stat, &outbuf);
       maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
       gss_release_buffer (&min_stat, &outbuf);
