@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,10 +28,12 @@ mu_m_server_t server;
 unsigned int idle_timeout;
 int imap4d_transcript;
 
-mu_mailbox_t mbox;
-char *homedir;
-int state = STATE_NONAUTH;
-struct mu_auth_data *auth_data;
+mu_mailbox_t mbox;              /* Current mailbox */
+char *real_homedir;             /* Homedir as returned by user database */
+char *imap4d_homedir;           /* Homedir as visible for the remote party */
+char *modify_homedir;           /* Expression to produce imap4d_homedir */
+int state = STATE_NONAUTH;      /* Current IMAP4 state */
+struct mu_auth_data *auth_data; 
 
 int login_disabled;             /* Disable LOGIN command */
 int tls_required;               /* Require STARTTLS */
@@ -62,25 +64,25 @@ static char doc[] = N_("GNU imap4d -- the IMAP4D daemon");
 #define OPT_FOREGROUND      260
 
 static struct argp_option options[] = {
-  { "foreground", OPT_FOREGROUND, 0, 0, N_("Remain in foreground."), 0},
-  { "inetd",  'i', 0, 0, N_("Run in inetd mode"), 0},
+  { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), 0},
+  { "inetd",  'i', 0, 0, N_("run in inetd mode"), 0},
   { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
-    N_("Runs in daemon mode with a maximum of NUMBER children"), 0 },
+    N_("runs in daemon mode with a maximum of NUMBER children"), 0 },
 
   {"other-namespace", 'O', N_("PATHLIST"), OPTION_HIDDEN,
-   N_("Set the `other' namespace"), 0},
+   N_("set the `other' namespace"), 0},
   {"shared-namespace", 'S', N_("PATHLIST"), OPTION_HIDDEN,
-   N_("Set the `shared' namespace"), 0},
+   N_("set the `shared' namespace"), 0},
   {"login-disabled", OPT_LOGIN_DISABLED, NULL, OPTION_HIDDEN,
-   N_("Disable LOGIN command")},
+   N_("disable LOGIN command")},
   {"create-home-dir", OPT_CREATE_HOME_DIR, N_("MODE"),
    OPTION_ARG_OPTIONAL|OPTION_HIDDEN,
-   N_("Create home directory, if it does not exist")},
+   N_("create home directory, if it does not exist")},
   {"preauth", OPT_PREAUTH, NULL, 0,
-   N_("Start in preauth mode") },
+   N_("start in preauth mode") },
 #ifdef WITH_TLS
   {"tls-required", OPT_TLS_REQUIRED, NULL, OPTION_HIDDEN,
-   N_("Always require STARTTLS before entering authentication phase")},
+   N_("always require STARTTLS before entering authentication phase")},
 #endif
   {NULL, 0, NULL, 0, NULL, 0}
 };
@@ -183,7 +185,7 @@ cb_mode (mu_debug_t debug, void *data, mu_config_value_t *val)
   home_dir_mode = strtoul (val->v.string, &p, 8);
   if (p[0] || (home_dir_mode & ~0777))
     mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-                         _("Invalid mode specification: %s"),
+                         _("invalid mode specification: %s"),
 			 val->v.string);
   return 0;
 }
@@ -255,8 +257,7 @@ cb_preauth (mu_debug_t debug, void *data, mu_config_value_t *val)
 
       if (rc)
 	{
-	  mu_cfg_format_error (debug, MU_DEBUG_ERROR,
-			       _("cannot create URL: %s"), mu_strerror (rc));
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_url_create", val->v.string, rc);
 	  return 1;
 	}
       rc = mu_url_parse (url);
@@ -297,6 +298,11 @@ cb_mailbox_mode (mu_debug_t debug, void *data, mu_config_value_t *val)
 }
 
 static struct mu_cfg_param imap4d_cfg_param[] = {
+  { "homedir", mu_cfg_string, &modify_homedir, 0, NULL,
+    N_("Modify home directory.") },
+  { "personal-namespace", MU_CFG_LIST_OF(mu_cfg_string), &namespace[NS_PRIVATE],
+    0, NULL, 
+    N_("Set personal namespace.") },
   { "other-namespace", MU_CFG_LIST_OF(mu_cfg_string), &namespace[NS_OTHER],
     0, NULL, 
     N_("Set other users' namespace.") },
@@ -342,17 +348,49 @@ static struct mu_cfg_param imap4d_cfg_param[] = {
 int
 imap4d_session_setup0 ()
 {
-  homedir = mu_normalize_path (mu_strdup (auth_data->dir));
-  if (imap4d_check_home_dir (homedir, auth_data->uid, auth_data->gid))
+  real_homedir = mu_normalize_path (mu_strdup (auth_data->dir));
+  if (imap4d_check_home_dir (real_homedir, auth_data->uid, auth_data->gid))
     return 1;
+
+  if (modify_homedir)
+    {
+      int rc;
+      mu_vartab_t vtab;
+      char *expr = mu_tilde_expansion (modify_homedir, "/", real_homedir);
+
+      mu_vartab_create (&vtab);
+      mu_vartab_define (vtab, "user", auth_data->name, 0);
+      mu_vartab_define (vtab, "home", real_homedir, 0);
+      rc = mu_vartab_expand (vtab, expr, &imap4d_homedir);
+      mu_vartab_destroy (&vtab);
+      free (expr);
+      if (rc)
+	{
+	  free (real_homedir);
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_vartab_expand",
+			   modify_homedir, rc);
+	  return 1;
+	}
+    }
+  else
+    imap4d_homedir = strdup (real_homedir);
+
+  if (strcmp (imap4d_homedir, real_homedir)
+      && imap4d_check_home_dir (imap4d_homedir,
+				auth_data->uid, auth_data->gid))
+    {
+      free (imap4d_homedir);
+      free (real_homedir);
+      return 1;
+    }
   
   if (auth_data->change_uid)
     setuid (auth_data->uid);
 
-  util_chdir (homedir);
-  namespace_init_session (homedir);
+  util_chdir (imap4d_homedir);
+  namespace_init_session (imap4d_homedir);
   mu_diag_output (MU_DIAG_INFO,
-		  _("User `%s' logged in (source: %s)"), auth_data->name,
+		  _("user `%s' logged in (source: %s)"), auth_data->name,
 		  auth_data->source);
   return 0;
 }
@@ -363,7 +401,7 @@ imap4d_session_setup (char *username)
   auth_data = mu_get_auth_by_name (username);
   if (auth_data == NULL)
     {
-      mu_diag_output (MU_DIAG_INFO, _("User `%s': nonexistent"), username);
+      mu_diag_output (MU_DIAG_INFO, _("user `%s' nonexistent"), username);
       return 1;
     }
   return imap4d_session_setup0 ();
@@ -376,9 +414,7 @@ get_client_address (int fd, struct sockaddr_in *pcs)
 
   if (getpeername (fd, (struct sockaddr *) pcs, &len) < 0)
     {
-      mu_diag_output (MU_DIAG_ERROR,
-		      _("Cannot obtain IP address of client: %s"),
-		      strerror (errno));
+      mu_diag_funcall (MU_DIAG_ERROR, "getpeername", NULL, errno);
       return 1;
     }
   return 0;
@@ -402,7 +438,7 @@ imap4d_mainloop (int fd, FILE *infile, FILE *outfile)
     {
       if (debug_mode)
 	{
-	  mu_diag_output (MU_DIAG_INFO, _("Started in debugging mode"));
+	  mu_diag_output (MU_DIAG_INFO, _("started in debugging mode"));
 	  text = "IMAP4rev1 Debugging mode";
 	}
       else
@@ -447,23 +483,23 @@ imap4d_check_home_dir (const char *dir, uid_t uid, gid_t gid)
 {
   struct stat st;
 
-  if (stat (homedir, &st))
+  if (stat (dir, &st))
     {
       if (errno == ENOENT && create_home_dir)
 	{
 	  mode_t mode = umask (0);
-	  int rc = mkdir (homedir, home_dir_mode);
+	  int rc = mkdir (dir, home_dir_mode);
 	  umask (mode);
 	  if (rc)
 	    {
 	      mu_error ("Cannot create home directory `%s': %s",
-			homedir, mu_strerror (errno));
+			dir, mu_strerror (errno));
 	      return 1;
 	    }
-	  if (chown (homedir, uid, gid))
+	  if (chown (dir, uid, gid))
 	    {
 	      mu_error ("Cannot set owner for home directory `%s': %s",
-			homedir, mu_strerror (errno));
+			dir, mu_strerror (errno));
 	      return 1;
 	    }
 	}
@@ -546,20 +582,19 @@ main (int argc, char **argv)
 	{
 	  if (errno == 0 || errno == ENOENT)
             {
-               mu_error (_("%s: No such group"), "mail");
+               mu_error (_("%s: no such group"), "mail");
                exit (EX_CONFIG);
             }
           else
             {
-	       mu_error (_("Error getting mail group: %s"), 
-                         mu_strerror (errno));
-	       exit (EX_OSERR);
+	      mu_diag_funcall (MU_DIAG_ERROR, "getgrnam", "mail", errno);
+	      exit (EX_OSERR);
             }
 	}
 
       if (setgid (gr->gr_gid) == -1)
 	{
-	  mu_error (_("Error setting mail group: %s"), mu_strerror (errno));
+	  mu_error (_("error setting mail group: %s"), mu_strerror (errno));
 	  exit (EX_OSERR);
 	}
     }
@@ -604,7 +639,7 @@ main (int argc, char **argv)
     }
 
   if (status)
-    mu_error (_("Main loop status: %s"), mu_strerror (status));	  
+    mu_error (_("main loop status: %s"), mu_strerror (status));	  
   /* Close the syslog connection and exit.  */
   closelog ();
 
