@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2008,
+   2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -18,37 +19,45 @@
 
 #include "imap4d.h"
 
+/*
+6.3.5.  RENAME Command
+
+   Arguments:  existing mailbox name
+               new mailbox name
+
+   Responses:  no specific responses for this command
+
+   Result:     OK - rename completed
+               NO - rename failure: can't rename mailbox with that name,
+                    can't rename to mailbox with that name
+               BAD - command unknown or arguments invalid
+*/  
 /*
   FIXME: Renaming a mailbox we must change the UIDVALIDITY
   of the mailbox.  */
 
 int
-imap4d_rename (struct imap4d_command *command, char *arg)
+imap4d_rename (struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   char *oldname;
   char *newname;
-  char *sp = NULL;
   int rc = RESP_OK;
   const char *msg = "Completed";
   struct stat newst;
   const char *delim = "/";
+  int ns;
+  
+  if (imap4d_tokbuf_argc (tok) != 4)
+    return util_finish (command, RESP_BAD, "Invalid arguments");
+  
+  oldname = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
+  newname = imap4d_tokbuf_getarg (tok, IMAP4_ARG_2);
 
-  oldname = util_getword (arg, &sp);
-  newname = util_getword (NULL, &sp);
-  if (!newname || !oldname)
-    return util_finish (command, RESP_BAD, "Too few arguments");
-
-  util_unquote (&newname);
-  util_unquote (&oldname);
-
-  if (*newname == '\0' || *oldname == '\0')
-    return util_finish (command, RESP_BAD, "Too few arguments");
-
-  if (strcasecmp (newname, "INBOX") == 0)
+  if (mu_c_strcasecmp (newname, "INBOX") == 0)
     return util_finish (command, RESP_NO, "Name Inbox is reservered");
 
   /* Allocates memory.  */
-  newname = namespace_getfullpath (newname, delim);
+  newname = namespace_getfullpath (newname, delim, &ns);
   if (!newname)
     return util_finish (command, RESP_NO, "Permission denied");
 
@@ -66,27 +75,24 @@ imap4d_rename (struct imap4d_command *command, char *arg)
   /* Renaming INBOX is permitted, and has special behavior.  It moves
      all messages in INBOX to a new mailbox with the given name,
      leaving INBOX empty.  */
-  if (strcasecmp (oldname, "INBOX") == 0)
+  if (mu_c_strcasecmp (oldname, "INBOX") == 0)
     {
       mu_mailbox_t newmbox = NULL;
       mu_mailbox_t inbox = NULL;
-      char *name;
 
-      if (S_ISDIR(newst.st_mode))
+      if (S_ISDIR (newst.st_mode))
 	{
 	  free (newname);
-	  return util_finish (command, RESP_NO, "Can not be a directory");
+	  return util_finish (command, RESP_NO, "Cannot be a directory");
 	}
-      name = calloc (strlen ("mbox:") + strlen (newname) + 1, 1);
-      sprintf (name, "mbox:%s", newname);
       if (mu_mailbox_create (&newmbox, newname) != 0
-	  || mu_mailbox_open (newmbox, MU_STREAM_CREAT | MU_STREAM_RDWR) != 0)
+	  || mu_mailbox_open (newmbox,
+			      MU_STREAM_CREAT | MU_STREAM_RDWR
+			        | mailbox_mode[ns]) != 0)
 	{
-	  free (name);
 	  free (newname);
-	  return util_finish (command, RESP_NO, "Can not create new mailbox");
+	  return util_finish (command, RESP_NO, "Cannot create new mailbox");
 	}
-      free (name);
       free (newname);
 
       if (mu_mailbox_create_default (&inbox, auth_data->name) == 0 &&
@@ -115,9 +121,12 @@ imap4d_rename (struct imap4d_command *command, char *arg)
       return util_finish (command, RESP_OK, "Already exist");
     }
 
-  oldname = namespace_getfullpath (oldname, delim);
+  oldname = namespace_getfullpath (oldname, delim, NULL);
 
   /* It must exist.  */
+  /* FIXME: 1. What if odlname or newname is a remote mailbox?
+            2. If newname is local and is in another namespace, its
+  	       permissions must be fixed */
   if (!oldname || rename (oldname, newname) != 0)
     {
       rc = RESP_NO;
