@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,20 +30,20 @@ static const char *sc2string (int);
 char *
 util_tilde_expansion (const char *ref, const char *delim)
 {
-  return mu_tilde_expansion (ref, delim, homedir);
+  return mu_tilde_expansion (ref, delim, imap4d_homedir);
 }
 
 /* Get the absolute path.  */
 /* NOTE: Path is allocated and must be free()d by the caller.  */
 char *
-util_getfullpath (char *name, const char *delim)
+util_getfullpath (const char *name, const char *delim)
 {
   char *p = util_tilde_expansion (name, delim);
   if (*p != delim[0])
     {
       char *s =
-	calloc (strlen (homedir) + strlen (delim) + strlen (p) + 1, 1);
-      sprintf (s, "%s%s%s", homedir, delim, p);
+	calloc (strlen (imap4d_homedir) + strlen (delim) + strlen (p) + 1, 1);
+      sprintf (s, "%s%s%s", imap4d_homedir, delim, p);
       free (p);
       p = s;
     }
@@ -121,6 +121,7 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
 	      {
 		if (*set)
 		  free (*set);
+		*set = NULL;
 		*n = 0;
 		return EINVAL;
 	      }
@@ -133,6 +134,7 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
 		  }
 		if (*set)
 		  free (*set);
+		*set = NULL;
 		*n = 0;
 		return EINVAL;
 	      }
@@ -147,6 +149,7 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
 		    if (tmp < 0 || val == 0)
 		      {
 			free (*set);
+			*set = NULL;
 			*n = 0;
 			return EINVAL;
 		      }
@@ -206,6 +209,7 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
 	  done = 1;
 	  if (*set)
 	    free (*set);
+	  *set = NULL;
 	  *n = 0;
 	  return EINVAL;
 
@@ -238,6 +242,7 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
 	  if (tmp < 0 || val == 0)
 	    {
 	      free (*set);
+	      *set = NULL;
 	      *n = 0;
 	      return EINVAL;
 	    }
@@ -340,7 +345,10 @@ util_out (int rc, const char *format, ...)
     imap4d_bye (ERR_NO_MEM);
 
   if (imap4d_transcript)
-    mu_diag_output (MU_DIAG_DEBUG, "sent: %s", buf);
+    {
+      int len = strcspn (buf, "\r\n");
+      mu_diag_output (MU_DIAG_DEBUG, "sent: %*.*s", len, len, buf);
+    }
 
   status = mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
@@ -381,7 +389,7 @@ util_finish (struct imap4d_command *command, int rc, const char *format, ...)
   free (tempbuf);
 
   if (imap4d_transcript)
-    mu_diag_output (MU_DIAG_DEBUG, "sent: %s\r\n", buf);
+    mu_diag_output (MU_DIAG_DEBUG, "sent: %s", buf);
 
   mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
@@ -444,16 +452,6 @@ util_do_command (imap4d_tokbuf_t tok)
   return command->func (command, tok);
 }
 
-int
-util_upper (char *s)
-{
-  if (!s)
-    return 0;
-  for (; *s; s++)
-    *s = toupper ((unsigned) *s);
-  return 0;
-}
-
 struct imap4d_command *
 util_getcommand (char *cmd, struct imap4d_command command_table[])
 {
@@ -462,7 +460,7 @@ util_getcommand (char *cmd, struct imap4d_command command_table[])
   for (i = 0; command_table[i].name != 0; i++)
     {
       if (strlen (command_table[i].name) == len &&
-	  !strcasecmp (command_table[i].name, cmd))
+	  !mu_c_strcasecmp (command_table[i].name, cmd))
 	return &command_table[i];
     }
   return NULL;
@@ -590,7 +588,7 @@ util_attribute_to_type (const char *item, int *type)
 {
   int i;
   for (i = 0; i < _imap4d_nattr; i++)
-    if (strcasecmp (item, _imap4d_attrlist[i].name) == 0)
+    if (mu_c_strcasecmp (item, _imap4d_attrlist[i].name) == 0)
       {
 	*type = _imap4d_attrlist[i].flag;
 	return 0;
@@ -675,86 +673,6 @@ util_attribute_matches_flag (mu_attribute_t attr, const char *item)
   return flags & mask;
 }
 
-int
-util_base64_encode (const unsigned char *input, size_t input_len,
-		    unsigned char **output, size_t * output_len)
-{
-  static char b64tab[] =
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-  size_t olen = 4 * (input_len + 2) / 3;
-  unsigned char *out = malloc (olen);
-
-  if (!out)
-    return 1;
-  *output = out;
-  while (input_len >= 3)
-    {
-      *out++ = b64tab[input[0] >> 2];
-      *out++ = b64tab[((input[0] << 4) & 0x30) | (input[1] >> 4)];
-      *out++ = b64tab[((input[1] << 2) & 0x3c) | (input[2] >> 6)];
-      *out++ = b64tab[input[2] & 0x3f];
-      olen -= 4;
-      input_len -= 3;
-      input += 3;
-    }
-
-  if (input_len > 0)
-    {
-      unsigned char c = (input[0] << 4) & 0x30;
-      *out++ = b64tab[input[0] >> 2];
-      if (input_len > 1)
-	c |= input[1] >> 4;
-      *out++ = b64tab[c];
-      *out++ = (input_len < 2) ? '=' : b64tab[(input[1] << 2) & 0x3c];
-      *out++ = '=';
-    }
-  *output_len = out - *output;
-  return 0;
-}
-
-int
-util_base64_decode (const unsigned char *input, size_t input_len,
-		    unsigned char **output, size_t * output_len)
-{
-  static int b64val[128] = {
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
-    52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1,
-    -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
-    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
-    -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
-    41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1
-  };
-  int olen = input_len;
-  unsigned char *out = malloc (olen);
-
-  if (!out)
-    return 1;
-  *output = out;
-  do
-    {
-      if (input[0] > 127 || b64val[input[0]] == -1
-	  || input[1] > 127 || b64val[input[1]] == -1
-	  || input[2] > 127 || ((input[2] != '=') && (b64val[input[2]] == -1))
-	  || input[3] > 127 || ((input[3] != '=')
-				&& (b64val[input[3]] == -1)))
-	return -1;
-      *out++ = (b64val[input[0]] << 2) | (b64val[input[1]] >> 4);
-      if (input[2] != '=')
-	{
-	  *out++ = ((b64val[input[1]] << 4) & 0xf0) | (b64val[input[2]] >> 2);
-	  if (input[3] != '=')
-	    *out++ = ((b64val[input[2]] << 6) & 0xc0) | b64val[input[3]];
-	}
-      input += 4;
-      input_len -= 4;
-    }
-  while (input_len > 0);
-  *output_len = out - *output;
-  return 0;
-}
-
 char *
 util_localname ()
 {
@@ -777,7 +695,7 @@ util_localname ()
 	}
       if (status || name == NULL)
 	{
-	  mu_diag_output (MU_DIAG_CRIT, _("Cannot find out my own hostname"));
+	  mu_diag_output (MU_DIAG_CRIT, _("cannot find out my own hostname"));
 	  exit (EX_OSERR);
 	}
 
@@ -882,13 +800,16 @@ util_uidvalidity (mu_mailbox_t smbox, unsigned long *uidvp)
 void
 util_setio (FILE *in, FILE *out)
 {
-  if (!out || !in)
+  if (!in)
+    imap4d_bye (ERR_NO_IFILE);
+  if (!out)
     imap4d_bye (ERR_NO_OFILE);
 
   setvbuf (in, NULL, _IOLBF, 0);
   setvbuf (out, NULL, _IOLBF, 0);
-  if (mu_stdio_stream_create (&istream, in, MU_STREAM_NO_CLOSE)
-      || mu_stdio_stream_create (&ostream, out, MU_STREAM_NO_CLOSE))
+  if (mu_stdio_stream_create (&istream, in, MU_STREAM_NO_CLOSE))
+    imap4d_bye (ERR_NO_IFILE);
+  if (mu_stdio_stream_create (&ostream, out, MU_STREAM_NO_CLOSE))
     imap4d_bye (ERR_NO_OFILE);
 }
 
@@ -932,8 +853,8 @@ util_wait_input (int timeout)
   status = mu_stream_wait (istream, &wflags, &tv);
   if (status)
     {
-      mu_diag_output (MU_DIAG_ERROR, _("Cannot poll input stream: %s"),
-	      mu_strerror(status));
+      mu_diag_output (MU_DIAG_ERROR, _("cannot poll input stream: %s"),
+		      mu_strerror(status));
       return -1;
     }
   return wflags & MU_STREAM_READY_RD;
@@ -966,7 +887,7 @@ imap4d_init_tls_server ()
     {
       const char *p;
       mu_stream_strerror (stream, &p);
-      mu_diag_output (MU_DIAG_ERROR, _("Cannot open TLS stream: %s"), p);
+      mu_diag_output (MU_DIAG_ERROR, _("cannot open TLS stream: %s"), p);
       return 0;
     }
 
@@ -1074,12 +995,12 @@ util_run_events (int old_state, int new_state)
 }
   
 void
-util_chdir (const char *homedir)
+util_chdir (const char *dir)
 {
-  int rc = chdir (homedir);
+  int rc = chdir (dir);
   if (rc)
     mu_error ("Cannot change to home directory `%s': %s",
-	      homedir, mu_strerror (errno));
+	      dir, mu_strerror (errno));
 }
 
 int
@@ -1089,7 +1010,7 @@ is_atom (const char *s)
     return 0;
   for (; *s; s++)
     {
-      if (*(const unsigned char *)s > 127 || iscntrl (*s))
+      if (mu_iscntrl (*s))
 	return 0;
     }
   return 1;
@@ -1215,8 +1136,7 @@ imap4d_tokbuf_expand (struct imap4d_tokbuf *tok, size_t size)
     }
 }
 
-#define ISDELIM(c) (strchr (".()[]<>", (c)) != NULL)
-#define ISWS(c) (c == ' ' || c == '\t')
+#define ISDELIM(c) (strchr ("()", (c)) != NULL)
 
 int
 util_isdelim (const char *str)
@@ -1242,7 +1162,7 @@ gettok (struct imap4d_tokbuf *tok, size_t off)
 {
   char *buf = tok->buffer;
   
-  while (off < tok->level && ISWS (buf[off]))
+  while (off < tok->level && mu_isblank (buf[off]))
     off++;
 
   if (tok->argc == tok->argmax)
@@ -1251,7 +1171,7 @@ gettok (struct imap4d_tokbuf *tok, size_t off)
 	tok->argmax = 16;
       else
 	tok->argmax *= 2;
-      tok->argp = realloc(tok->argp, tok->argmax * sizeof (tok->argp[0]));
+      tok->argp = realloc (tok->argp, tok->argmax * sizeof (tok->argp[0]));
       if (!tok->argp)
 	imap4d_bye (ERR_NO_MEM);
     }
@@ -1272,7 +1192,7 @@ gettok (struct imap4d_tokbuf *tok, size_t off)
 	{
 	  size_t len;
 	  off++;
-	  len  = unquote(buf + off, p - (buf + off));
+	  len  = unquote (buf + off, p - (buf + off));
 	  buf[off + len] = 0;
 	  tok->argp[tok->argc++] = off;
 	  return p - buf + 1;
@@ -1283,7 +1203,7 @@ gettok (struct imap4d_tokbuf *tok, size_t off)
   if (ISDELIM (buf[off]))
     return insert_nul (tok, off + 1);
 
-  while (off < tok->level && !ISWS (buf[off]))
+  while (off < tok->level && !mu_isblank (buf[off]))
     {
       if (ISDELIM (buf[off]))
 	return insert_nul (tok, off);
@@ -1311,13 +1231,13 @@ check_input_err (int rc, size_t sz)
 	p = mu_strerror (rc);
       
       mu_diag_output (MU_DIAG_INFO,
-		      _("Error reading from input file: %s"), p);
-      imap4d_bye (ERR_NO_OFILE);
+		      _("error reading from input file: %s"), p);
+      imap4d_bye (ERR_NO_IFILE);
     }
   else if (sz == 0)
     {
-      mu_diag_output (MU_DIAG_INFO, _("Unexpected eof on input"));
-      imap4d_bye (ERR_NO_OFILE);
+      mu_diag_output (MU_DIAG_INFO, _("unexpected eof on input"));
+      imap4d_bye (ERR_NO_IFILE);
     }
 }
 
@@ -1358,8 +1278,28 @@ imap4d_readline (struct imap4d_tokbuf *tok)
       char *last_arg;
       size_t off = imap4d_tokbuf_getline (tok);
       if (transcript)
-        mu_diag_output (MU_DIAG_DEBUG, "recv: %s", tok->buffer);
+        {
+          int len;
+          char *p = mu_strcasestr (tok->buffer, "LOGIN");
+          if (p && p > tok->buffer && mu_isblank (p[-1]))
+            {
+	      char *q = mu_str_skip_class (p + 5, MU_CTYPE_SPACE);
+	      q = mu_str_skip_class_comp (q, MU_CTYPE_SPACE);
+              len = q - tok->buffer; 
+              mu_diag_output (MU_DIAG_DEBUG,
+			      "recv: %*.*s {censored}", len, len,
+                              tok->buffer);
+             }
+           else
+             {
+               len = strcspn (tok->buffer, "\r\n");
+               mu_diag_output (MU_DIAG_DEBUG, "recv: %*.*s", 
+                               len, len, tok->buffer);
+             }
+        }
       imap4d_tokbuf_tokenize (tok, off);
+      if (tok->argc == 0)
+        break;  
       last_arg = tok->buffer + tok->argp[tok->argc - 1];
       if (last_arg[0] == '{' && last_arg[strlen(last_arg)-1] == '}')
 	{
@@ -1437,7 +1377,17 @@ imap4d_getline (char **pbuf, size_t *psize, size_t *pnbytes)
       char *s = *pbuf;
       len = util_trim_nl (s, len);
       if (imap4d_transcript)
-        mu_diag_output (MU_DIAG_DEBUG, "recv: %s", s);
+        {
+          if (len)
+            mu_diag_output (MU_DIAG_DEBUG, "recv: %s", s);
+          else
+            mu_diag_output (MU_DIAG_DEBUG, "got EOF");
+        }
+      if (len == 0)
+        {
+          imap4d_bye (ERR_NO_IFILE);
+          /*FIXME rc = ECONNABORTED;*/
+        }
       if (pnbytes)
 	*pnbytes = len;
     }
