@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +22,7 @@
 #include <stdarg.h>
 
 #include <mailutils/types.h>
+#include <mailutils/diag.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -30,8 +31,8 @@ extern "C" {
 typedef int (*mu_error_pfn_t) (const char *fmt, va_list ap);
 
 extern int mu_verror (const char *fmt, va_list ap);
-extern int mu_error (const char *fmt, ...);
-extern void mu_error_set_print (mu_error_pfn_t);
+extern int mu_error (const char *fmt, ...) MU_PRINTFLIKE(1,2);
+extern void mu_error_set_print (mu_error_pfn_t) __attribute__((deprecated));
 
 int mu_default_error_printer (const char *fmt, va_list ap);
 int mu_syslog_error_printer (const char *fmt, va_list ap);
