@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -66,14 +66,14 @@ typedef int (*mu_auth_fp) (struct mu_auth_data **data,
 
 struct mu_auth_module
 {
-  char           *name;
-  gocs_init_fp   init;
-  mu_auth_fp     authenticate;
-  void           *authenticate_data;
-  mu_auth_fp     auth_by_name;
-  void           *auth_by_name_data;
-  mu_auth_fp     auth_by_uid;
-  void           *auth_by_uid_data;
+  char            *name;
+  mu_gocs_init_fp init;
+  mu_auth_fp      authenticate;
+  void            *authenticate_data;
+  mu_auth_fp      auth_by_name;
+  void            *auth_by_name_data;
+  mu_auth_fp      auth_by_uid;
+  void            *auth_by_uid_data;
 };
 
 enum mu_auth_key_type
@@ -99,7 +99,7 @@ extern struct mu_auth_data *
 mu_get_auth_by_uid (uid_t uid);
 
 extern int
-mu_authenticate (struct mu_auth_data *auth_data, char *pass);
+mu_authenticate (struct mu_auth_data *auth_data, const char *pass);
 
 extern int mu_auth_nosupport (struct mu_auth_data **return_data,
 			      const void *key,
