@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2006, 2007, 
+   2009 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -53,10 +54,10 @@ static char *
 _get_envelope_sender (mu_envelope_t env)
 {
   mu_address_t addr;
-  char buffer[128];
+  const char *buffer;
   char *ptr;
   
-  if (mu_envelope_sender (env, buffer, sizeof (buffer), NULL)
+  if (mu_envelope_sget_sender (env, &buffer)
       || mu_address_create (&addr, buffer))
     return NULL;
 
@@ -75,11 +76,12 @@ mu_scm_message_print (SCM message_smob, SCM port, scm_print_state * pstate)
 {
   struct mu_message *mum = (struct mu_message *) SCM_CDR (message_smob);
   mu_envelope_t env = NULL;
-  char buffer[128];
+  const char *buffer;
   const char *p;
   size_t m_size = 0, m_lines = 0;
   struct tm tm;
   mu_timezone tz;
+  char datebuf[sizeof ("Mon Jan 01 00:00")]; /* Warning: length must be > 9 */
 
   mu_message_get_envelope (mum->msg, &env);
 
@@ -102,12 +104,14 @@ mu_scm_message_print (SCM message_smob, SCM port, scm_print_state * pstate)
       else
 	scm_puts ("UNKNOWN", port);
       
-      mu_envelope_date (env, buffer, sizeof (buffer), NULL);
-      p = buffer;
-      if (mu_parse_ctime_date_time (&p, &tm, &tz) == 0)
-	strftime (buffer, sizeof (buffer), "%a %b %e %H:%M", &tm);
+      if (mu_envelope_sget_date (env, &p) == 0
+          && mu_parse_ctime_date_time (&p, &tm, &tz) == 0)
+	{
+	  strftime (datebuf, sizeof (datebuf), "%a %b %e %H:%M", &tm);
+	  buffer = datebuf;
+	}
       else
-	strcpy (buffer, "UNKNOWN");
+	buffer = "UNKNOWN";
       scm_puts ("\" \"", port);
       scm_puts (buffer, port);
       scm_puts ("\" ", port);
@@ -115,9 +119,9 @@ mu_scm_message_print (SCM message_smob, SCM port, scm_print_state * pstate)
       mu_message_size (mum->msg, &m_size);
       mu_message_lines (mum->msg, &m_lines);
       
-      snprintf (buffer, sizeof (buffer), "%3lu %-5lu",
+      snprintf (datebuf, sizeof (datebuf), "%3lu %-5lu",
 		(unsigned long) m_lines, (unsigned long) m_size);
-      scm_puts (buffer, port);
+      scm_puts (datebuf, port);
     }
   scm_puts (">", port);
   return 1;
@@ -422,7 +426,7 @@ string_sloppy_member (SCM lst, char *name)
     {
       SCM car = SCM_CAR (lst);
       if (scm_is_string (car)
-	  && strcasecmp (scm_i_string_chars (car), name) == 0)
+	  && mu_c_strcasecmp (scm_i_string_chars (car), name) == 0)
 	return 1;
     }
   return 0;
@@ -1001,7 +1005,7 @@ SCM_DEFINE (scm_mu_message_send, "mu-message-send", 1, 3, 0,
     {
       mu_debug_t debug = NULL;
       mu_mailer_get_debug (mailer, &debug);
-      mu_debug_set_level (debug, MU_DEBUG_TRACE|MU_DEBUG_PROT);
+      mu_debug_set_level (debug, MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
     }
 
   status = mu_mailer_open (mailer, MU_STREAM_RDWR);
