@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,8 +30,10 @@ char *quotadbname = NULL;  /* Name of mailbox quota database */
 char *quota_query = NULL;  /* SQL query to retrieve mailbox quota */
 
 char *sender_address = NULL;       
-char *progfile_pattern = NULL;
-char *sieve_pattern = NULL;
+
+maidag_script_fun script_handler;
+
+mu_list_t script_list;
 
 char *forward_file = NULL;
 int forward_file_checks = FWD_ALL;
@@ -54,7 +56,7 @@ int reuse_lmtp_address = 1;
 
 const char *program_version = "maidag (" PACKAGE_STRING ")";
 static char doc[] =
-N_("GNU maildag -- the mail delivery agent")
+N_("GNU maidag -- the mail delivery agent")
 "\v"
 N_("Debug flags are:\n\
   g - guimb stack traces\n\
@@ -73,28 +75,40 @@ static char args_doc[] = N_("[recipient...]");
 
 static struct argp_option options[] = 
 {
-  { "foreground", FOREGROUND_OPTION, 0, 0, N_("Remain in foreground."), 0 },
-  { "inetd",  'i', 0, 0, N_("Run in inetd mode"), 0 },
+#define GRID 0
+ { NULL, 0, NULL, 0,
+   N_("General options"), GRID },
+      
+  { "foreground", FOREGROUND_OPTION, 0, 0, N_("remain in foreground"),
+    GRID + 1 },
+  { "inetd",  'i', 0, 0, N_("run in inetd mode"), GRID + 1 },
   { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
-    N_("Runs in daemon mode with a maximum of NUMBER children"), 0 },
-  { "url", URL_OPTION, 0, 0, N_("Deliver to given URLs"), 0 },
+    N_("runs in daemon mode with a maximum of NUMBER children"), GRID + 1 },
+  { "url", URL_OPTION, 0, 0, N_("deliver to given URLs"), GRID + 1 },
   { "from", 'f', N_("EMAIL"), 0,
-    N_("Specify the sender's name") },
+    N_("specify the sender's name"), GRID + 1 },
   { NULL, 'r', NULL, OPTION_ALIAS, NULL },
-  { "sieve", 'S', N_("PATTERN"), 0,
-    N_("Set name pattern for user-defined Sieve mail filters"), 0 },
-  { "message-id-header", MESSAGE_ID_HEADER_OPTION, N_("STRING"), 0,
-    N_("Identify messages by the value of this header when logging Sieve actions"), 0 },
-#ifdef WITH_GUILE
-  { "source", 's', N_("PATTERN"), 0,
-    N_("Set name pattern for user-defined Scheme mail filters"), 0 },
-#endif
   { "lmtp", LMTP_OPTION, N_("URL"), OPTION_ARG_OPTIONAL,
-    N_("Operate in LMTP mode"), 0 },
+    N_("operate in LMTP mode"), GRID + 1 },
   { "debug", 'x', N_("FLAGS"), 0,
-    N_("Enable debugging"), 0 },
+    N_("enable debugging"), GRID + 1 },
   { "stderr", STDERR_OPTION, NULL, 0,
-    N_("Log to standard error"), 0 },
+    N_("log to standard error"), GRID + 1 },
+#undef GRID
+
+#define GRID 2
+ { NULL, 0, NULL, 0,
+   N_("Scripting options"), GRID },
+ 
+  { "language", 'l', N_("STRING"), 0,
+    N_("define scripting language for the next --script option"),
+    GRID + 1 },
+  { "script", 's', N_("PATTERN"), 0,
+    N_("set name pattern for user-defined mail filter"), GRID + 1 },
+  { "message-id-header", MESSAGE_ID_HEADER_OPTION, N_("STRING"), 0,
+    N_("use this header to identify messages when logging Sieve actions"),
+    GRID + 1 },
+#undef GRID
   { NULL,      0, NULL, 0, NULL, 0 }
 };
 
@@ -128,7 +142,7 @@ set_debug_flags (mu_debug_t debug, const char *arg)
 {
   while (*arg)
     {
-      if (isdigit (*arg))
+      if (mu_isdigit (*arg))
 	debug_level = strtoul (arg, (char**)&arg, 10);
       else
 	for (; *arg && *arg != ','; arg++)
@@ -201,21 +215,30 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 'r':
     case 'f':
       if (sender_address != NULL)
-	{
-	  argp_error (state, _("Multiple --from options"));
-	  return EX_USAGE;
-	}
+	argp_error (state, _("multiple --from options"));
       sender_address = arg;
       break;
       
-#ifdef WITH_GUILE	
-    case 's':
-      mu_argp_node_list_new (&lst, "guile-filter", arg);
+    case 'l':
+      script_handler = script_lang_handler (arg);
+      if (!script_handler)
+	argp_error (state, _("unknown or unsupported language: %s"),
+		    arg);
       break;
-#endif
+      
+    case 's':
+      switch (script_register (arg))
+	{
+	case 0:
+	  break;
 
-    case 'S':
-      mu_argp_node_list_new (&lst, "sieve-filter", arg);
+	case EINVAL:
+	  argp_error (state, _("%s has unknown file suffix"), arg);
+	  break;
+
+	default:
+	  argp_error (state, _("error registering script"));
+	}
       break;
       
     case 'x':
@@ -268,7 +291,7 @@ cb2_group (mu_debug_t debug, const char *gname, void *data)
     mu_list_create (plist);
   group = getgrnam (gname);
   if (!group)
-    mu_cfg_format_error (debug, MU_DEBUG_ERROR, _("Unknown group: %s"), gname);
+    mu_cfg_format_error (debug, MU_DEBUG_ERROR, _("unknown group: %s"), gname);
   else
     mu_list_append (*plist, (void*)group->gr_gid);
   return 0;
@@ -302,7 +325,7 @@ cb2_forward_file_checks (mu_debug_t debug, const char *name, void *data)
   const char *str;
   int val;
   
-  if (strlen (name) > 2 && strncasecmp (name, "no", 2) == 0)
+  if (strlen (name) > 2 && mu_c_strncasecmp (name, "no", 2) == 0)
     {
       negate = 1;
       str = name + 2;
@@ -311,7 +334,7 @@ cb2_forward_file_checks (mu_debug_t debug, const char *name, void *data)
     str = name;
 
   if (mu_kwd_xlat_name_ci (forward_checks, str, &val))
-    mu_cfg_format_error (debug, MU_DEBUG_ERROR, _("Unknown keyword: %s"),
+    mu_cfg_format_error (debug, MU_DEBUG_ERROR, _("unknown keyword: %s"),
 			 name);
   else
     {
@@ -329,6 +352,54 @@ cb_forward_file_checks (mu_debug_t debug, void *data, mu_config_value_t *arg)
   return mu_cfg_string_value_cb (debug, arg, cb2_forward_file_checks, data);
 }
 
+static int
+cb_script_language (mu_debug_t debug, void *data, mu_config_value_t *val)
+{
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  script_handler = script_lang_handler (val->v.string);
+  if (!script_handler)
+    {
+      mu_cfg_format_error (debug, MU_DEBUG_ERROR,
+			   _("unsupported language: %s"),
+			   val->v.string);
+      return 1;
+    }
+  return 0;
+}
+
+static int
+cb_script_pattern (mu_debug_t debug, void *data, mu_config_value_t *val)
+{
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  
+  switch (script_register (val->v.string))
+    {
+    case 0:
+      break;
+
+    case EINVAL:
+      mu_cfg_format_error (debug, MU_DEBUG_ERROR,
+			   _("%s has unknown file suffix"),
+			   val->v.string);
+      break;
+
+    default:
+      mu_cfg_format_error (debug, MU_DEBUG_ERROR,
+			   _("error registering script"));
+    }
+  return 0;
+}
+
+struct mu_cfg_param filter_cfg_param[] = {
+  { "language", mu_cfg_callback, NULL, 0, cb_script_language,
+    N_("Set script language.") },
+  { "pattern", mu_cfg_callback, NULL, 0, cb_script_pattern,
+    N_("Set script pattern.") },
+  { NULL }
+};
+    
 struct mu_cfg_param maidag_cfg_param[] = {
   { "exit-multiple-delivery-success", mu_cfg_bool, &multiple_delivery, 0, NULL,
     N_("In case of multiple delivery, exit with code 0 if at least one "
@@ -347,18 +418,10 @@ struct mu_cfg_param maidag_cfg_param[] = {
        "sql { ... } instead."),
     N_("query") },
 #endif
-  { "sieve-filter", mu_cfg_string, &sieve_pattern, 0, NULL,
-    N_("File name or name pattern for Sieve filter file."),
-    N_("file-or-pattern") },
   { "message-id-header", mu_cfg_string, &message_id_header, 0, NULL,
     N_("When logging Sieve actions, identify messages by the value of "
        "this header."),
     N_("name") },
-#ifdef WITH_GUILE
-  { "guile-filter", mu_cfg_string, &progfile_pattern, 0, NULL,
-    N_("File name or name pattern for Guile filter file."),
-    N_("file-or-pattern") },
-#endif
   { "debug", mu_cfg_callback, NULL, 0, cb_debug,
     N_("Set maidag debug level.  Debug level consists of one or more "
        "of the following letters:\n"
@@ -388,145 +451,37 @@ struct mu_cfg_param maidag_cfg_param[] = {
     N_("url") },
   { "reuse-address", mu_cfg_bool, &reuse_lmtp_address, 0, NULL,
     N_("Reuse existing address (LMTP mode).  Default is \"yes\".") },
+  { "filter", mu_cfg_section, NULL, 0, NULL,
+    N_("Add a message filter") },
   { ".server", mu_cfg_section, NULL, 0, NULL,
     N_("LMTP server configuration.") },
   TCP_WRAPPERS_CONFIG
   { NULL }
 };
 
-
-/* Logging */
-
-static int
-_sieve_debug_printer (void *unused, const char *fmt, va_list ap)
-{
-  mu_diag_vprintf (MU_DIAG_DEBUG, fmt, ap);
-  return 0;
-}
-
 static void
-_sieve_action_log (void *user_name,
-		   const mu_sieve_locus_t *locus, size_t msgno,
-		   mu_message_t msg,
-		   const char *action, const char *fmt, va_list ap)
+maidag_cfg_init ()
 {
-  int pfx = 0;
-  mu_debug_t debug;
-
-  mu_diag_get_debug (&debug);
-  mu_debug_set_locus (debug, locus->source_file, locus->source_line);
-  
-  mu_diag_printf (MU_DIAG_NOTICE, _("(user %s) "), (char*) user_name);
-  if (message_id_header)
+  struct mu_cfg_section *section;
+  if (mu_create_canned_section ("filter", &section) == 0)
     {
-      mu_header_t hdr = NULL;
-      char *val = NULL;
-      mu_message_get_header (msg, &hdr);
-      if (mu_header_aget_value (hdr, message_id_header, &val) == 0
-	  || mu_header_aget_value (hdr, MU_HEADER_MESSAGE_ID, &val) == 0)
-	{
-	  pfx = 1;
-	  mu_diag_printf (MU_DIAG_NOTICE, _("%s on msg %s"), action, val);
-	  free (val);
-	}
+      section->docstring = N_("Add new message filter.");
+      mu_cfg_section_add_params (section, filter_cfg_param);
     }
-  
-  if (!pfx)
-    {
-      size_t uid = 0;
-      mu_message_get_uid (msg, &uid);
-      mu_diag_printf (MU_DIAG_NOTICE, _("%s on msg uid %d"), action, uid);
-    }
-  
-  if (fmt && strlen (fmt))
-    {
-      mu_diag_printf (MU_DIAG_NOTICE, "; ");
-      mu_diag_vprintf (MU_DIAG_NOTICE, fmt, ap);
-    }
-  mu_diag_printf (MU_DIAG_NOTICE, "\n");
-  mu_debug_set_locus (debug, NULL, 0);
 }
 
-static int
-_sieve_parse_error (void *user_name, const char *filename, int lineno,
-		    const char *fmt, va_list ap)
-{
-  mu_debug_t debug;
-
-  mu_diag_get_debug (&debug);
-  if (filename)
-    mu_debug_set_locus (debug, filename, lineno);
-
-  mu_diag_printf (MU_DIAG_ERROR, _("(user %s) "), (char*) user_name);
-  mu_diag_vprintf (MU_DIAG_ERROR, fmt, ap);
-  mu_diag_printf (MU_DIAG_ERROR, "\n");
-  mu_debug_set_locus (debug, NULL, 0);
-  return 0;
-}
-
-int
-sieve_test (struct mu_auth_data *auth, mu_message_t msg)
-{
-  int rc = 1;
-  char *progfile;
-    
-  if (!sieve_pattern)
-    return 1;
-
-  progfile = mu_expand_path_pattern (sieve_pattern, auth->name);
-  if (access (progfile, R_OK))
-    {
-      if (debug_level > 2)
-	mu_diag_output (MU_DIAG_DEBUG, _("Access to %s failed: %m"), progfile);
-    }
-  else
-    {
-      mu_sieve_machine_t mach;
-      rc = mu_sieve_machine_init (&mach, auth->name);
-      if (rc)
-	{
-	  mu_error (_("Cannot initialize sieve machine: %s"),
-		    mu_strerror (rc));
-	}
-      else
-	{
-	  mu_sieve_set_debug (mach, _sieve_debug_printer);
-	  mu_sieve_set_debug_level (mach, sieve_debug_flags);
-	  mu_sieve_set_parse_error (mach, _sieve_parse_error);
-	  if (sieve_enable_log)
-	    mu_sieve_set_logger (mach, _sieve_action_log);
-	  
-	  rc = mu_sieve_compile (mach, progfile);
-	  if (rc == 0)
-	    {
-	      mu_attribute_t attr;
-
-	      mu_message_get_attribute (msg, &attr);
-	      mu_attribute_unset_deleted (attr);
-	      if (switch_user_id (auth, 1) == 0)
-		{
-		  chdir (auth->dir);
-		
-		  rc = mu_sieve_message (mach, msg);
-		  if (rc == 0)
-		    rc = mu_attribute_is_deleted (attr) == 0;
-
-		  switch_user_id (auth, 0);
-		  chdir ("/");
-		}
-	      mu_sieve_machine_destroy (&mach);
-	    }
-	}
-    }
-  free (progfile);
-  return rc;
-}
+/* FIXME: These are for compatibility with MU 2.0.
+   Remove in 2.2 */
+extern mu_record_t mu_remote_smtp_record;
+extern mu_record_t mu_remote_sendmail_record;
+extern mu_record_t mu_remote_prog_record;
 
 
 int
 main (int argc, char *argv[])
 {
   int arg_index;
+  mu_debug_t debug;
 
   /* Preparative work: close inherited fds, force a reasonable umask
      and prepare a logging. */
@@ -549,14 +504,18 @@ main (int argc, char *argv[])
   mu_register_all_formats ();
   mu_registrar_record (mu_smtp_record);
 
-  /* Register a special mailbox */
-  mu_registrar_record (mu_remote_mbox_record);
+  /* FIXME: These are for compatibility with MU 2.0.
+     Remove in 2.1 */
+  mu_registrar_record (mu_remote_smtp_record);
+  mu_registrar_record (mu_remote_sendmail_record);
+  mu_registrar_record (mu_remote_prog_record);
   
   mu_gocs_register ("sieve", mu_sieve_module_init);
 
   mu_tcpwrapper_cfg_init ();
   mu_acl_cfg_init ();
   mu_m_server_cfg_init ();
+  maidag_cfg_init ();
   
   /* Parse command line */
   mu_argp_init (program_version, NULL);
@@ -577,16 +536,18 @@ main (int argc, char *argv[])
   if (log_to_stderr == -1)
     log_to_stderr = url_option || (!lmtp_mode && (current_uid != 0));
   
+  mu_diag_get_debug (&debug);
   if (!log_to_stderr)
     {
-      mu_debug_t debug;
-
       openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
-      mu_diag_get_debug (&debug);
       mu_debug_set_print (debug, mu_diag_syslog_printer, NULL);
-
       mu_debug_default_printer = mu_debug_syslog_printer;
     }
+  else
+    {
+      mu_debug_set_print (debug, mu_diag_stderr_printer, NULL);
+      mu_debug_default_printer = mu_debug_stderr_printer;
+    }
 
   argc -= arg_index;
   argv += arg_index;
@@ -595,7 +556,7 @@ main (int argc, char *argv[])
     {
       if (argc)
 	{
-	  mu_error (_("Too many arguments"));
+	  mu_error (_("too many arguments"));
 	  return EX_USAGE;
 	}
       return maidag_lmtp_server ();
@@ -615,13 +576,13 @@ main (int argc, char *argv[])
 	      
 	      if (!current_uid)
 		{
-		  mu_error (_("Cannot get username"));
+		  mu_error (_("cannot get username"));
 		  return EX_UNAVAILABLE;
 		}
 	      
 	      if (argc > 0 && strcmp (auth->name, argv[0]))
 		{
-		  mu_error (_("Recipients given when running as non-root"));
+		  mu_error (_("recipients given when running as non-root"));
 		  return EX_USAGE;
 		}
 	      s_argv[0] = auth->name;
