@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2003, 2005, 2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -26,8 +26,8 @@
 int
 mail_pipe (int argc, char **argv)
 {
-  message_t msg;
-  stream_t stream;
+  mu_message_t msg;
+  mu_stream_t stream;
   char *cmd;
   FILE *tube;
   msgset_t *list, *mp;
@@ -35,9 +35,9 @@ mail_pipe (int argc, char **argv)
   off_t off = 0;
   size_t n = 0;
 
-  if (argc > 1)
+  if (argc > 2)
     cmd = argv[--argc];
-  else if (util_getenv (&cmd, "cmd", Mail_env_boolean, 1))
+  else if (mailvar_get (&cmd, "cmd", mailvar_type_string, 1))
     return 1;
 
   if (msgset_parse (argc, argv, MSG_NODELETED|MSG_SILENT, &list))
@@ -49,16 +49,16 @@ mail_pipe (int argc, char **argv)
     {
       if (util_get_message (mbox, mp->msg_part[0], &msg) == 0)
 	{
-	  message_get_stream (msg, &stream);
+	  mu_message_get_stream (msg, &stream);
 	  off = 0;
-	  while (stream_read (stream, buffer, sizeof (buffer) - 1, off,
+	  while (mu_stream_read (stream, buffer, sizeof (buffer) - 1, off,
 			      &n) == 0 && n != 0)
 	    {
 	      buffer[n] = '\0';
 	      fprintf (tube, "%s", buffer);
 	      off += n;
 	    }
-	  if (util_getenv (NULL, "page", Mail_env_boolean, 0) == 0)
+	  if (mailvar_get (NULL, "page", mailvar_type_boolean, 0) == 0)
 	    fprintf (tube, "\f\n");
 	}
       util_mark_read (msg);
