@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 
+   2007, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -38,7 +39,9 @@ mail_print_msg (msgset_t *mspec, mu_message_t mesg, void *data)
   int pagelines = util_get_crt ();
   
   mu_message_lines (mesg, &lines);
-
+  if (mailvar_get (NULL, "showenvelope", mailvar_type_boolean, 0) == 0)
+    lines++;
+  
   /* If it is POP or IMAP the lines number is not known, so try
      to be smart about it.  */
   if (lines == 0)
@@ -58,10 +61,13 @@ mail_print_msg (msgset_t *mspec, mu_message_t mesg, void *data)
   if (pagelines && lines > pagelines)
     out = popen (getenv ("PAGER"), "w");
 
+  if (mailvar_get (NULL, "showenvelope", mailvar_type_boolean, 0) == 0)
+    print_envelope (mspec, mesg, "From");
+  
   if (*(int *) data) /* print was called with a lowercase 'p' */
     {
       size_t i, num = 0;
-      char buf[512];
+      const char *sptr;
       char *tmp;
       
       mu_message_get_header (mesg, &hdr);
@@ -69,12 +75,13 @@ mail_print_msg (msgset_t *mspec, mu_message_t mesg, void *data)
 
       for (i = 1; i <= num; i++)
 	{
-	  mu_header_get_field_name (hdr, i, buf, sizeof buf, NULL);
-	  if (mail_header_is_visible (buf))
+	  if (mu_header_sget_field_name (hdr, i, &sptr))
+	    continue;
+	  if (mail_header_is_visible (sptr))
 	    {
-	      fprintf (out, "%s: ", buf);
+	      fprintf (out, "%s: ", sptr);
 	      mu_header_aget_field_value (hdr, i, &tmp);
-	      if (mail_header_is_unfoldable (buf))
+	      if (mail_header_is_unfoldable (sptr))
 		mu_string_unfold (tmp, NULL);
 	      util_rfc2047_decode (&tmp);
 	      fprintf (out, "%s\n", tmp);
@@ -113,7 +120,7 @@ mail_print_msg (msgset_t *mspec, mu_message_t mesg, void *data)
 int
 mail_print (int argc, char **argv)
 {
-  int lower = islower (argv[0][0]);
+  int lower = mu_islower (argv[0][0]);
   int rc = util_foreach_msg (argc, argv, MSG_NODELETED|MSG_SILENT,
 			     mail_print_msg, &lower);
   return rc;
