@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -25,18 +25,15 @@
    mbox upon normal termination. */
 
 static int
-touch0 (msgset_t *mspec, message_t msg, void *data)
+touch0 (msgset_t *mspec, mu_message_t msg, void *data)
 {
-  attribute_t attr = NULL;
+  mu_attribute_t attr = NULL;
   
-  message_get_attribute (msg, &attr);
-  if (!attribute_is_userflag (attr, MAIL_ATTRIBUTE_SAVED))
-    {
-      attribute_set_userflag (attr, MAIL_ATTRIBUTE_MBOXED);
-      util_mark_read (msg);
-    }
+  mu_message_get_attribute (msg, &attr);
+  mu_attribute_set_userflag (attr, MAIL_ATTRIBUTE_TOUCHED);
   
-  cursor = mspec->msg_part[0];
+  set_cursor (mspec->msg_part[0]);
+  return 0;
 }
 
 int
