@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2004, 2005,
-   2007, 2008 Free Software Foundation, Inc.
+   2007, 2008, 2009 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,7 @@
 #include <mailutils/address.h>
 #include <mailutils/mutil.h>
 #include <mailutils/errno.h>
+#include <mailutils/cstr.h>
 
 #include <header0.h>
 
@@ -79,13 +80,13 @@ mu_hdrent_find (struct _mu_header *hdr, const char *name, int pos)
   if (pos > 0)
     {
       for (p = hdr->head; p; p = p->next)
-	if (strcasecmp (MU_HDRENT_NAME (hdr,p), name) == 0 && pos-- == 1)
+	if (mu_c_strcasecmp (MU_HDRENT_NAME (hdr,p), name) == 0 && pos-- == 1)
 	  break;
     }
   else if (pos < 0)
     {
       for (p = hdr->tail; p; p = p->prev)
-	if (strcasecmp (MU_HDRENT_NAME (hdr,p), name) == 0 && ++pos == 0)
+	if (mu_c_strcasecmp (MU_HDRENT_NAME (hdr,p), name) == 0 && ++pos == 0)
 	  break;
     }
   else
@@ -761,9 +762,7 @@ mu_header_get_address_n (mu_header_t header, const char *name, int n,
   if (status)
     return status;
 
-  status = mu_address_create(addr, value);
-
-  return status;
+  return mu_address_create (addr, value);
 }
 
 
