@@ -109,11 +109,11 @@
 #line 1 "parsedate.y"
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
+   the Free Software Foundation; either version 2, or (at your option)
    any later version.
    
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -123,8 +123,7 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
   
 /* A heavily modified version of the well-known public domain getdate.y.
    It was originally written by Steven M. Bellovin <smb@research.att.com>
@@ -135,9 +134,6 @@
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
-# ifdef HAVE_ALLOCA_H
-#  include <alloca.h>
-# endif
 #endif
 
 #include <stdio.h>
@@ -239,19 +235,36 @@ struct pd_date
 };
 
 #define DATE_INIT(date) memset(&(date), 0, sizeof(date))
-#define DATE_SET(date, memb, m, val) \
- do { date . memb = val; date.mask |= m; } while (0)
-
-#define SET_SECOND(d,v)   DATE_SET(d,second,PD_MASK_SECOND,v)
-#define SET_MINUTE(d,v)   DATE_SET(d,minute,PD_MASK_MINUTE,v)  
-#define SET_HOUR(d,v)     DATE_SET(d,hour,PD_MASK_HOUR,v)
-#define SET_DAY(d,v)      DATE_SET(d,day,PD_MASK_DAY,v)   
-#define SET_MONTH(d,v)    DATE_SET(d,month,PD_MASK_MONTH,v)
-#define SET_YEAR(d,v)     DATE_SET(d,year,PD_MASK_YEAR,v)  
-#define SET_TZ(d,v)       DATE_SET(d,tz,PD_MASK_TZ,v)
-#define SET_MERIDIAN(d,v) DATE_SET(d,meridian,PD_MASK_MERIDIAN,v)
-#define SET_ORDINAL(d,v)  DATE_SET(d,ordinal,PD_MASK_ORDINAL,v)
-#define SET_NUMBER(d,v)   DATE_SET(d,number,PD_MASK_NUMBER,v) 
+#define DATE_SET(date, memb, m, val, lim, onerror)                        \
+ do                                                                       \
+   {                                                                      \
+     int __x = val;                                                       \
+     if (((m) != PD_MASK_TZ && __x < 0) || (lim && __x > lim)) onerror;   \
+     date . memb = __x; date.mask |= m;                                   \
+   }                                                                      \
+ while (0)
+   
+#define __SET_SECOND(d,v,a)   DATE_SET(d,second,PD_MASK_SECOND,v,59,a)
+#define __SET_MINUTE(d,v,a)   DATE_SET(d,minute,PD_MASK_MINUTE,v,59,a)  
+#define __SET_HOUR(d,v,a)     DATE_SET(d,hour,PD_MASK_HOUR,v,23,a)
+#define __SET_DAY(d,v,a)      DATE_SET(d,day,PD_MASK_DAY,v,31,a)   
+#define __SET_MONTH(d,v,a)    DATE_SET(d,month,PD_MASK_MONTH,v,12,a)
+#define __SET_YEAR(d,v,a)     DATE_SET(d,year,PD_MASK_YEAR,v,0,a)  
+#define __SET_TZ(d,v,a)       DATE_SET(d,tz,PD_MASK_TZ,v,0,a)
+#define __SET_MERIDIAN(d,v,a) DATE_SET(d,meridian,PD_MASK_MERIDIAN,v,MER24,a)
+#define __SET_ORDINAL(d,v,a)  DATE_SET(d,ordinal,PD_MASK_ORDINAL,v,0,a)
+#define __SET_NUMBER(d,v,a)   DATE_SET(d,number,PD_MASK_NUMBER,v,0,a) 
+ 
+#define SET_SECOND(d,v)   __SET_SECOND(d,v,YYERROR)   
+#define SET_MINUTE(d,v)   __SET_MINUTE(d,v,YYERROR)   
+#define SET_HOUR(d,v)     __SET_HOUR(d,v,YYERROR)     
+#define SET_DAY(d,v)      __SET_DAY(d,v,YYERROR)      
+#define SET_MONTH(d,v)    __SET_MONTH(d,v,YYERROR)    
+#define SET_YEAR(d,v)     __SET_YEAR(d,v,YYERROR)     
+#define SET_TZ(d,v)       __SET_TZ(d,v,YYERROR)       
+#define SET_MERIDIAN(d,v) __SET_MERIDIAN(d,v,YYERROR) 
+#define SET_ORDINAL(d,v)  __SET_ORDINAL(d,v,YYERROR)  
+#define SET_NUMBER(d,v)   __SET_NUMBER(d,v,YYERROR)   
 
 int
 pd_date_union (struct pd_date *a, struct pd_date *b)
@@ -327,7 +340,7 @@ static const char	*pd_yyinput;
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 201 "parsedate.y"
+#line 214 "parsedate.y"
 {
   int number;
   enum meridian meridian;
@@ -335,7 +348,7 @@ typedef union YYSTYPE
   struct pd_datespec datespec;
 }
 /* Line 187 of yacc.c.  */
-#line 339 "parsedate.c"
+#line 352 "parsedate.c"
 	YYSTYPE;
 # define pd_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -348,7 +361,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 352 "parsedate.c"
+#line 365 "parsedate.c"
 
 #ifdef short
 # undef short
@@ -652,12 +665,12 @@ static const pd_yytype_int8 pd_yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const pd_yytype_uint16 pd_yyrline[] =
 {
-       0,   222,   222,   229,   233,   239,   245,   276,   277,   278,
-     279,   282,   288,   295,   305,   313,   326,   331,   336,   343,
-     349,   355,   363,   369,   389,   397,   414,   420,   427,   433,
-     440,   455,   465,   469,   475,   483,   488,   493,   498,   503,
-     508,   513,   518,   523,   528,   533,   538,   543,   548,   553,
-     558,   563,   568,   576,   579
+       0,   235,   235,   242,   246,   252,   258,   289,   290,   291,
+     292,   295,   301,   308,   319,   327,   340,   345,   350,   357,
+     363,   369,   377,   383,   403,   411,   428,   434,   441,   447,
+     454,   469,   479,   483,   489,   497,   502,   507,   512,   517,
+     522,   527,   532,   537,   542,   547,   552,   557,   562,   567,
+     572,   577,   582,   590,   593
 };
 #endif
 
@@ -1604,14 +1617,14 @@ pd_yyreduce:
   switch (pd_yyn)
     {
         case 2:
-#line 223 "parsedate.y"
+#line 236 "parsedate.y"
     {
 	    pd = (pd_yyvsp[(1) - (1)].datespec);
 	  }
     break;
 
   case 3:
-#line 229 "parsedate.y"
+#line 242 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.datespec).date);
 	    DATE_INIT ((pd_yyval.datespec).rel);
@@ -1619,7 +1632,7 @@ pd_yyreduce:
     break;
 
   case 4:
-#line 234 "parsedate.y"
+#line 247 "parsedate.y"
     {
 	    if (pd_date_union (&(pd_yyvsp[(1) - (2)].datespec).date, &(pd_yyvsp[(2) - (2)].date)))
 	      YYERROR;
@@ -1628,7 +1641,7 @@ pd_yyreduce:
     break;
 
   case 5:
-#line 240 "parsedate.y"
+#line 253 "parsedate.y"
     {
 	    if (pd_date_union (&(pd_yyvsp[(1) - (2)].datespec).rel, &(pd_yyvsp[(2) - (2)].date)))
 	      YYERROR;
@@ -1637,7 +1650,7 @@ pd_yyreduce:
     break;
 
   case 6:
-#line 246 "parsedate.y"
+#line 259 "parsedate.y"
     {
 	    if (MASK_IS_SET ((pd_yyvsp[(1) - (2)].datespec).date.mask, (PD_MASK_TIME|PD_MASK_DATE))
 		&& !(pd_yyvsp[(1) - (2)].datespec).rel.mask)
@@ -1669,7 +1682,7 @@ pd_yyreduce:
     break;
 
   case 11:
-#line 283 "parsedate.y"
+#line 296 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number));
@@ -1678,7 +1691,7 @@ pd_yyreduce:
     break;
 
   case 12:
-#line 289 "parsedate.y"
+#line 302 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (4)].number));
@@ -1688,7 +1701,7 @@ pd_yyreduce:
     break;
 
   case 13:
-#line 296 "parsedate.y"
+#line 309 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (4)].number));
@@ -1697,11 +1710,12 @@ pd_yyreduce:
 	    SET_TZ ((pd_yyval.date), ((pd_yyvsp[(4) - (4)].number) < 0
 			   ? -(pd_yyvsp[(4) - (4)].number) % 100 + (-(pd_yyvsp[(4) - (4)].number) / 100) * 60
 			   : - ((pd_yyvsp[(4) - (4)].number) % 100 + ((pd_yyvsp[(4) - (4)].number) / 100) * 60)));
+
 	  }
     break;
 
   case 14:
-#line 306 "parsedate.y"
+#line 320 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (6)].number));
@@ -1712,7 +1726,7 @@ pd_yyreduce:
     break;
 
   case 15:
-#line 314 "parsedate.y"
+#line 328 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (6)].number));
@@ -1726,7 +1740,7 @@ pd_yyreduce:
     break;
 
   case 16:
-#line 327 "parsedate.y"
+#line 341 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_TZ ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number));
@@ -1734,7 +1748,7 @@ pd_yyreduce:
     break;
 
   case 17:
-#line 332 "parsedate.y"
+#line 346 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_TZ ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number) - 60);
@@ -1742,7 +1756,7 @@ pd_yyreduce:
     break;
 
   case 18:
-#line 337 "parsedate.y"
+#line 351 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_TZ ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) - 60);
@@ -1750,7 +1764,7 @@ pd_yyreduce:
     break;
 
   case 19:
-#line 344 "parsedate.y"
+#line 358 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_ORDINAL ((pd_yyval.date), 1);
@@ -1759,7 +1773,7 @@ pd_yyreduce:
     break;
 
   case 20:
-#line 350 "parsedate.y"
+#line 364 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_ORDINAL ((pd_yyval.date), 1);
@@ -1768,7 +1782,7 @@ pd_yyreduce:
     break;
 
   case 21:
-#line 356 "parsedate.y"
+#line 370 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_ORDINAL ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number));
@@ -1777,7 +1791,7 @@ pd_yyreduce:
     break;
 
   case 22:
-#line 364 "parsedate.y"
+#line 378 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(1) - (3)].number));
@@ -1786,7 +1800,7 @@ pd_yyreduce:
     break;
 
   case 23:
-#line 370 "parsedate.y"
+#line 384 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    /* Interpret as YYYY/MM/DD if $1 >= 1000, otherwise as MM/DD/YY.
@@ -1809,7 +1823,7 @@ pd_yyreduce:
     break;
 
   case 24:
-#line 390 "parsedate.y"
+#line 404 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    /* ISO 8601 format.  pd_yypd_yy-mm-dd.  */
@@ -1820,7 +1834,7 @@ pd_yyreduce:
     break;
 
   case 25:
-#line 398 "parsedate.y"
+#line 412 "parsedate.y"
     {
 	    /* either 17-JUN-1992 or 1992-JUN-17 */
 	    DATE_INIT ((pd_yyval.date));
@@ -1840,7 +1854,7 @@ pd_yyreduce:
     break;
 
   case 26:
-#line 415 "parsedate.y"
+#line 429 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number));
@@ -1849,7 +1863,7 @@ pd_yyreduce:
     break;
 
   case 27:
-#line 421 "parsedate.y"
+#line 435 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(1) - (4)].number));
@@ -1859,7 +1873,7 @@ pd_yyreduce:
     break;
 
   case 28:
-#line 428 "parsedate.y"
+#line 442 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(2) - (2)].number));
@@ -1868,7 +1882,7 @@ pd_yyreduce:
     break;
 
   case 29:
-#line 434 "parsedate.y"
+#line 448 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(2) - (3)].number));
@@ -1878,7 +1892,7 @@ pd_yyreduce:
     break;
 
   case 30:
-#line 441 "parsedate.y"
+#line 455 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 
@@ -1894,7 +1908,7 @@ pd_yyreduce:
     break;
 
   case 31:
-#line 456 "parsedate.y"
+#line 470 "parsedate.y"
     {
 	    (pd_yyvsp[(1) - (2)].date).second = - (pd_yyvsp[(1) - (2)].date).second;
 	    (pd_yyvsp[(1) - (2)].date).minute = - (pd_yyvsp[(1) - (2)].date).minute;
@@ -1907,7 +1921,7 @@ pd_yyreduce:
     break;
 
   case 33:
-#line 470 "parsedate.y"
+#line 484 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    if (pd_date_union (&(pd_yyval.date), &(pd_yyvsp[(1) - (1)].date)))
@@ -1916,7 +1930,7 @@ pd_yyreduce:
     break;
 
   case 34:
-#line 476 "parsedate.y"
+#line 490 "parsedate.y"
     {
 	    if (pd_date_union (&(pd_yyvsp[(1) - (2)].date), &(pd_yyvsp[(2) - (2)].date)))
 	      YYERROR;
@@ -1925,7 +1939,7 @@ pd_yyreduce:
     break;
 
   case 35:
-#line 484 "parsedate.y"
+#line 498 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_YEAR ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -1933,7 +1947,7 @@ pd_yyreduce:
     break;
 
   case 36:
-#line 489 "parsedate.y"
+#line 503 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_YEAR ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -1941,7 +1955,7 @@ pd_yyreduce:
     break;
 
   case 37:
-#line 494 "parsedate.y"
+#line 508 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_YEAR ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number));
@@ -1949,7 +1963,7 @@ pd_yyreduce:
     break;
 
   case 38:
-#line 499 "parsedate.y"
+#line 513 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -1957,7 +1971,7 @@ pd_yyreduce:
     break;
 
   case 39:
-#line 504 "parsedate.y"
+#line 518 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -1965,7 +1979,7 @@ pd_yyreduce:
     break;
 
   case 40:
-#line 509 "parsedate.y"
+#line 523 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MONTH ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number));
@@ -1973,7 +1987,7 @@ pd_yyreduce:
     break;
 
   case 41:
-#line 514 "parsedate.y"
+#line 528 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_DAY ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -1981,7 +1995,7 @@ pd_yyreduce:
     break;
 
   case 42:
-#line 519 "parsedate.y"
+#line 533 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_DAY ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -1989,7 +2003,7 @@ pd_yyreduce:
     break;
 
   case 43:
-#line 524 "parsedate.y"
+#line 538 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_DAY ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number));
@@ -1997,7 +2011,7 @@ pd_yyreduce:
     break;
 
   case 44:
-#line 529 "parsedate.y"
+#line 543 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -2005,7 +2019,7 @@ pd_yyreduce:
     break;
 
   case 45:
-#line 534 "parsedate.y"
+#line 548 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -2013,7 +2027,7 @@ pd_yyreduce:
     break;
 
   case 46:
-#line 539 "parsedate.y"
+#line 553 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_HOUR ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number));
@@ -2021,7 +2035,7 @@ pd_yyreduce:
     break;
 
   case 47:
-#line 544 "parsedate.y"
+#line 558 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MINUTE ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -2029,7 +2043,7 @@ pd_yyreduce:
     break;
 
   case 48:
-#line 549 "parsedate.y"
+#line 563 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MINUTE ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -2037,7 +2051,7 @@ pd_yyreduce:
     break;
 
   case 49:
-#line 554 "parsedate.y"
+#line 568 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_MINUTE ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number));
@@ -2045,7 +2059,7 @@ pd_yyreduce:
     break;
 
   case 50:
-#line 559 "parsedate.y"
+#line 573 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_SECOND ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -2053,7 +2067,7 @@ pd_yyreduce:
     break;
 
   case 51:
-#line 564 "parsedate.y"
+#line 578 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_SECOND ((pd_yyval.date), (pd_yyvsp[(1) - (2)].number) * (pd_yyvsp[(2) - (2)].number));
@@ -2061,7 +2075,7 @@ pd_yyreduce:
     break;
 
   case 52:
-#line 569 "parsedate.y"
+#line 583 "parsedate.y"
     {
 	    DATE_INIT ((pd_yyval.date));
 	    SET_SECOND ((pd_yyval.date), (pd_yyvsp[(1) - (1)].number));
@@ -2069,14 +2083,14 @@ pd_yyreduce:
     break;
 
   case 53:
-#line 576 "parsedate.y"
+#line 590 "parsedate.y"
     {
 	    (pd_yyval.meridian) = MER24;
 	  }
     break;
 
   case 54:
-#line 580 "parsedate.y"
+#line 594 "parsedate.y"
     {
 	    (pd_yyval.meridian) = (pd_yyvsp[(1) - (1)].meridian);
 	  }
@@ -2084,7 +2098,7 @@ pd_yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 2088 "parsedate.c"
+#line 2102 "parsedate.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", pd_yyr1[pd_yyn], &pd_yyval, &pd_yyloc);
@@ -2298,7 +2312,7 @@ pd_yyreturn:
 }
 
 
-#line 585 "parsedate.y"
+#line 599 "parsedate.y"
 
 
 #include <mailutils/types.h>
@@ -2777,19 +2791,19 @@ mu_parse_date (const char *p, time_t *rettime, const time_t *now)
     return -1;
   
   if (!MASK_IS_SET (pd.date.mask, PD_MASK_YEAR))
-    SET_YEAR (pd.date, tmp->tm_year + TM_YEAR_ORIGIN);
+    __SET_YEAR (pd.date, tmp->tm_year + TM_YEAR_ORIGIN, return -1);
   if (!MASK_IS_SET (pd.date.mask, PD_MASK_MONTH))
-    SET_MONTH (pd.date, tmp->tm_mon + 1);
+    __SET_MONTH (pd.date, tmp->tm_mon + 1, return -1);
   if (!MASK_IS_SET (pd.date.mask, PD_MASK_DAY))
-    SET_DAY (pd.date, tmp->tm_mday);
+    __SET_DAY (pd.date, tmp->tm_mday, return -1);
   if (!MASK_IS_SET (pd.date.mask, PD_MASK_HOUR))
-    SET_HOUR (pd.date, tmp->tm_hour);
+    __SET_HOUR (pd.date, tmp->tm_hour, return -1);
   if (!MASK_IS_SET (pd.date.mask, PD_MASK_MERIDIAN))
-    SET_MERIDIAN (pd.date, MER24);
+    __SET_MERIDIAN (pd.date, MER24, return -1);
   if (!MASK_IS_SET (pd.date.mask, PD_MASK_MINUTE))
-    SET_MINUTE (pd.date, tmp->tm_min);
+    __SET_MINUTE (pd.date, tmp->tm_min, return -1);
   if (!MASK_IS_SET (pd.date.mask, PD_MASK_SECOND))
-    SET_SECOND (pd.date, tmp->tm_sec);
+    __SET_SECOND (pd.date, tmp->tm_sec, return -1);
   
   tm.tm_year = norm_year (pd.date.year) - TM_YEAR_ORIGIN + pd.rel.year;
   tm.tm_mon = pd.date.month - 1 + pd.rel.month;
@@ -2895,8 +2909,7 @@ main (int argc, char *argv[])
   buff[MAX_BUFF_LEN] = 0;
   while (fgets (buff, MAX_BUFF_LEN, stdin) && buff[0])
     {
-      d = get_date (buff, (time_t *) NULL);
-      if (d == -1)
+      if (mu_parse_date (buff, &d, NULL))
 	printf ("Bad format - couldn't convert.\n");
       else
 	printf ("%s", ctime (&d));
