@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2004, 2005,
-   2006, 2007 Free Software Foundation, Inc.
+   2006, 2007, 2009 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -40,6 +40,7 @@
 
 #include <mailutils/property.h>
 #include <mailutils/errno.h>
+#include <mailutils/io.h>
 #include <stream0.h>
 
 static int refill (mu_stream_t, mu_off_t);
@@ -297,7 +298,7 @@ mu_stream_readline (mu_stream_t is, char *buf, size_t count,
       size_t n, nr = 0;
       char c;
       /* Grossly inefficient hopefully they override this */
-      for (n = 1; n < count; n++)
+      for (n = 0; n < count; )
 	{
 	  status = is->_read (is, &c, 1, offset, &nr);
 	  if (status != 0) /* Error.  */
@@ -306,19 +307,16 @@ mu_stream_readline (mu_stream_t is, char *buf, size_t count,
 	    {
 	      *buf++ = c;
 	      offset++;
+	      n++;
 	      if (c == '\n') /* Newline is stored like fgets().  */
 		break;
 	    }
 	  else if (nr == 0)
-	    {
-	      if (n == 1) /* EOF, no data read.  */
-		n = 0;
-	      break; /* EOF, some data was read.  */
-	    }
+	    break; /* EOF */
 	}
       *buf = '\0';
       if (pnread)
-	*pnread = (n == count) ? n - 1: n;
+	*pnread = n;
     }
   else /* Buffered.  */
     {
@@ -493,43 +491,16 @@ mu_stream_write (mu_stream_t os, const char *buf, size_t count,
 int
 mu_stream_vprintf (mu_stream_t os, mu_off_t *poff, const char *fmt, va_list ap)
 {
-  char *buf, *p;
-  size_t buflen = 512;
+  char *buf = NULL, *p;
+  size_t buflen = 0;
   size_t n;
-  int done = 0;
   int rc;
 
-  buf = calloc (1, buflen);
-  if (buf == NULL)
-    return ENOMEM;
-
-  do
-    {
-      n = vsnprintf (buf, buflen, fmt, ap);
-      if (n < 0 || n >= buflen || !memchr (buf, '\0', n + 1))
-	{
-	  char *newbuf;
-	  size_t newlen = buflen * 2;
-	  if (newlen < buflen)
-	    {
-	      free (buf);
-	      return ENOMEM;
-	    }
-	  newbuf = realloc (buf, newlen);
-	  if (newbuf == NULL)
-	    {
-	      free (buf);
-	      return ENOMEM;
-	    }
-	  buflen = newlen;
-	  buf = newbuf;
-	}
-      else
-	done = 1;
-    }
-  while (!done);
-
+  rc = mu_vasnprintf (&buf, &buflen, fmt, ap);
+  if (rc)
+    return rc;
   p = buf;
+  n = strlen (buf);
   do
     {
       size_t wrs;
