@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -22,6 +22,11 @@
 #include <mailutils/diag.h>
 #include <mailutils/kwd.h>
 #include <mailutils/syslog.h>
+#include <mailutils/cstr.h>
+
+#ifndef LOG_AUTHPRIV
+# define LOG_AUTHPRIV
+#endif
 
 static mu_kwd_t kw_facility[] = {
   { "USER",    LOG_USER },   
@@ -44,7 +49,7 @@ static mu_kwd_t kw_facility[] = {
 static int
 syslog_to_n (mu_kwd_t *kw, const char *str, int *pint)
 {
-  if (strncasecmp (str, "LOG_", 4) == 0)
+  if (mu_c_strncasecmp (str, "LOG_", 4) == 0)
     str += 4;
   return mu_kwd_xlat_name_ci (kw, str, pint);
 }
