@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2009 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -22,9 +22,12 @@
 
 #include <mailutils/nls.h>
 #include <mailutils/version.h>
+#include <mailutils/cstr.h>
 #include <stdio.h>
 #include <string.h>
 
+#include <confpaths.h>
+
 char *mu_license_text =
  N_("   GNU Mailutils is free software; you can redistribute it and/or modify\n"
     "   it under the terms of the GNU General Public License as published by\n"
@@ -67,6 +70,9 @@ static struct mu_conf_option mu_conf_option[] = {
 #ifdef WITH_GDBM
   { "WITH_GDBM", N_("GNU DBM") },
 #endif
+#ifdef WITH_TOKYOCABINET
+  { "WITH_TOKYOCABINET", N_("Tokyo Cabinet DBM") },
+#endif
 #ifdef WITH_GNUTLS
   { "WITH_GNUTLS", N_("TLS support using GNU TLS") },
 #endif
@@ -79,6 +85,9 @@ static struct mu_conf_option mu_conf_option[] = {
 #ifdef WITH_GUILE
   { "WITH_GUILE", N_("Support for Guile as extension language") },
 #endif
+#ifdef WITH_PYTHON
+  { "WITH_PYTHON", N_("Support for Python as extension language") },
+#endif
 #ifdef WITH_PTHREAD
   { "WITH_PTHREAD", N_("Support for POSIX threads") },
 #endif
@@ -168,10 +177,10 @@ mu_check_option (char *name)
       else
 	len = strlen (mu_conf_option[i].name);
 
-      if (strncasecmp (mu_conf_option[i].name, name, len) == 0)
+      if (mu_c_strncasecmp (mu_conf_option[i].name, name, len) == 0)
 	return &mu_conf_option[i];
       else if ((q = strchr (mu_conf_option[i].name, '_')) != NULL
-	       && strncasecmp (q + 1, name,
+	       && mu_c_strncasecmp (q + 1, name,
 			       len - (q - mu_conf_option[i].name) - 1) == 0)
 	return &mu_conf_option[i];
     }
