@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -24,33 +24,31 @@
 #include <obstack.h>
 
 const char *program_version = "burst (" PACKAGE_STRING ")";
-/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
-   in this message */
-static char doc[] = N_("GNU MH burst\v\
-Options marked with `*' are not yet implemented.\n\
+static char doc[] = N_("GNU MH burst")"\v"
+N_("Options marked with `*' are not yet implemented.\n\
 Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = "[msgs]";
 
 /* GNU options */
 static struct argp_option options[] = {
   {"folder",        ARG_FOLDER,        N_("FOLDER"), 0,
-   N_("Specify folder to operate upon")},
+   N_("specify folder to operate upon")},
   {"inplace",      ARG_INPLACE,      N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Replace the source message with the table of contents, insert extracted messages after it") },
+   N_("replace the source message with the table of contents, insert extracted messages after it") },
   {"noinplace",    ARG_NOINPLACE,    0, OPTION_HIDDEN, ""},
   {"quiet",        ARG_QUIET,        N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Be quiet about the messages that are not in digest format") },
+   N_("be quiet about the messages that are not in digest format") },
   {"noquiet",      ARG_NOQUIET,      0, OPTION_HIDDEN, ""},
   {"verbose",      ARG_VERBOSE,      N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Verbosely list the actions taken") },
+   N_("verbosely list the actions taken") },
   {"noverbose",    ARG_NOVERBOSE,    0, OPTION_HIDDEN, ""},
   {"recursive",    ARG_RECURSIVE,    N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Recursively expand MIME messages") },
+   N_("recursively expand MIME messages") },
   {"norecursive",  ARG_NORECURSIVE,  0, OPTION_HIDDEN, ""},
   {"length",       ARG_LENGTH,       N_("NUMBER"), 0,
-   N_("Set minimal length of digest encapsulation boundary (default 1)") },
+   N_("set minimal length of digest encapsulation boundary (default 1)") },
   {"license",      ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
   { NULL }
 };
 
@@ -77,7 +75,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
   switch (key)
     {
     case ARG_FOLDER: 
-      current_folder = arg;
+      mh_set_current_folder (arg);
       break;
 
     case ARG_INPLACE:
@@ -482,7 +480,7 @@ burst_rename (mh_msgset_t *ms, size_t lastuid)
 }  
 
 void
-msg_copy (size_t num, char *file)
+msg_copy (size_t num, const char *file)
 {
   mu_message_t msg;
   mu_attribute_t attr = NULL;
@@ -547,20 +545,20 @@ main (int argc, char **argv)
   int index, rc;
   mu_mailbox_t mbox;
   mh_msgset_t msgset;
-  char *tempfolder = mh_global_profile_get ("Temp-Folder", ".temp");
+  const char *tempfolder = mh_global_profile_get ("Temp-Folder", ".temp");
   
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
   argc -= index;
   argv += index;
 
-  VERBOSE ((_("Opening folder `%s'"), current_folder));
-  mbox = mh_open_folder (current_folder, 0);
+  VERBOSE ((_("Opening folder `%s'"), mh_current_folder ()));
+  mbox = mh_open_folder (mh_current_folder (), 0);
   mh_msgset_parse (mbox, &msgset, argc, argv, "cur");
 
   if (inplace)
@@ -595,7 +593,7 @@ main (int argc, char **argv)
       size_t i, next_uid, last_uid;
       mh_msgset_t ms;
       char *xargv[2];
-      char *dir;
+      const char *dir;
       
       burst_map = obstack_finish (&stk);
 
