@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2005, 2006 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -29,29 +29,29 @@ static char args_doc[] = N_("[+folder]");
 /* GNU options */
 static struct argp_option options[] = {
   {"file",    ARG_FILE, N_("FILE"),   0,
-   N_("Incorporate mail from named file")},
+   N_("incorporate mail from named file")},
   {"folder",  ARG_FOLDER, N_("FOLDER"), 0,
-   N_("Specify folder to incorporate mail to")},
+   N_("specify folder to incorporate mail to")},
   {"audit",   ARG_AUDIT, N_("FILE"), 0,
-   N_("Enable audit")},
+   N_("enable audit")},
   {"noaudit", ARG_NOAUDIT, 0, 0,
-   N_("Disable audit")},
+   N_("disable audit")},
   {"changecur", ARG_CHANGECUR, N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Mark first incorporated message as current (default)")},
+   N_("mark first incorporated message as current (default)")},
   {"nochangecur", ARG_NOCHANGECUR, NULL, OPTION_HIDDEN, ""},
   {"form",    ARG_FORM, N_("FILE"),   0,
-   N_("Read format from given file")},
+   N_("read format from given file")},
   {"format",  ARG_FORMAT, N_("FORMAT"), 0,
-   N_("Use this format string")},
+   N_("use this format string")},
   {"truncate", ARG_TRUNCATE, N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Truncate source mailbox after incorporating (default)")},
+   N_("truncate source mailbox after incorporating (default)")},
   {"notruncate", ARG_NOTRUNCATE, NULL, OPTION_HIDDEN, ""},
   {"width",   ARG_WIDTH, N_("NUMBER"), 0,
-   N_("Set output width")},
+   N_("set output width")},
   {"quiet",   ARG_QUIET, 0,        0,
-   N_("Be quiet")},
+   N_("be quiet")},
   {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
   { 0 }
 };
 
@@ -77,7 +77,7 @@ static FILE *audit_fp;
 static int changecur = -1;
 static int truncate_source = -1;
 static int quiet = 0;
-static char *append_folder;
+static const char *append_folder;
 
 static int
 opt_handler (int key, char *arg, void *unused, struct argp_state *state)
@@ -133,7 +133,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       width = strtoul (arg, NULL, 0);
       if (!width)
 	{
-	  argp_error (state, _("Invalid width"));
+	  argp_error (state, _("invalid width"));
 	  exit (1);
 	}
       break;
@@ -179,9 +179,9 @@ main (int argc, char **argv)
   int rc;
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, NULL);
 
@@ -196,7 +196,7 @@ main (int argc, char **argv)
     {
       if ((rc = mu_mailbox_create_default (&input, NULL)) != 0)
 	{
-	  mu_error (_("Cannot create default mailbox"),
+	  mu_error (_("cannot create default mailbox: %s"),
 		    mu_strerror (rc));
 	  exit (1);
 	}
@@ -205,7 +205,7 @@ main (int argc, char **argv)
     }
   else if ((rc = mu_mailbox_create_default (&input, input_file)) != 0)
     {
-      mu_error (_("Cannot create mailbox %s: %s"),
+      mu_error (_("cannot create mailbox %s: %s"),
 		input_file, mu_strerror (rc));
       exit (1);
     }
@@ -214,7 +214,7 @@ main (int argc, char **argv)
     {
       mu_url_t url;
       mu_mailbox_get_url (input, &url);
-      mu_error (_("Cannot open mailbox %s: %s"),
+      mu_error (_("cannot open mailbox %s: %s"),
 		mu_url_to_string (url),
 		mu_strerror (errno));
       exit (1);
@@ -222,14 +222,14 @@ main (int argc, char **argv)
 
   if ((rc = mu_mailbox_messages_count (input, &total)) != 0)
     {
-      mu_error (_("Cannot read input mailbox: %s"), mu_strerror (errno));
+      mu_error (_("cannot read input mailbox: %s"), mu_strerror (errno));
       exit (1);
     }
 
   output = mh_open_folder (append_folder, 1);
   if ((rc = mu_mailbox_messages_count (output, &lastmsg)) != 0)
     {
-      mu_error (_("Cannot read output mailbox: %s"),
+      mu_error (_("cannot read output mailbox: %s"),
 		mu_strerror (errno));
       exit (1);
     }
