@@ -104,7 +104,7 @@
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2004,
-   2007 Free Software Foundation, Inc.
+   2007, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1504,7 +1504,7 @@ fmt_yyreduce:
   case 13:
 #line 138 "mh_fmtgram.y"
     {
-	      if (strcasecmp ((fmt_yyvsp[(3) - (4)].str), "body") == 0)
+	      if (mu_c_strcasecmp ((fmt_yyvsp[(3) - (4)].str), "body") == 0)
 		{
 		  mh_code_op (mhop_body);
 		}
@@ -1989,7 +1989,7 @@ fmt_yylex ()
     {
       curp++;
       percent = 1;
-      if (isdigit (*curp) || *curp == '-')
+      if (mu_isdigit (*curp) || *curp == '-')
 	{
 	  int num = 0;
 	  int flags = 0;
@@ -2001,7 +2001,7 @@ fmt_yylex ()
 	    }
 	  if (*curp == '0')
 	    flags |= MH_FMT_ZEROPAD;
-	  while (*curp && isdigit (*curp))
+	  while (*curp && mu_isdigit (*curp))
 	    num = num * 10 + *curp++ - '0';
 	  fmt_yylval.num = num | flags;
 	  return FMTSPEC;
@@ -2087,7 +2087,7 @@ fmt_yylex ()
 	{
 	  curp -= rest;
 	  fmt_yylval.builtin = bp;
-	  while (*curp && isspace(*curp))
+	  while (*curp && mu_isspace (*curp))
 	    curp++;
 	  return FUNCTION;
 	}
@@ -2245,7 +2245,7 @@ mh_code_builtin (mh_builtin_t *bp, int argtype)
 	    }
 	  else
 	    {
-	      mu_error (_("Missing argument for %s"), bp->name);
+	      mu_error (_("missing argument for %s"), bp->name);
 	      return 0;
 	    }
 	}
@@ -2254,7 +2254,7 @@ mh_code_builtin (mh_builtin_t *bp, int argtype)
 	  switch (bp->argtype)
 	    {
 	    case mhtype_none:
-	      mu_error (_("Extra arguments to %s"), bp->name);
+	      mu_error (_("extra arguments to %s"), bp->name);
 	      return 0;
 	      
 	    case mhtype_num:
