@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2005 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -20,46 +20,13 @@
 /* This module implements execution of MH format strings. */
 
 #include <mh.h>
+#include <mh_format.h>
 #include <mailutils/mime.h>
 
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
 
-#define obstack_chunk_alloc malloc
-#define obstack_chunk_free free
-#include <obstack.h>
-
-typedef struct       /* A string object type */
-{
-  int size;          /* Allocated size or 0 for static storage */
-  char *ptr;         /* Actual data */
-} strobj_t;
-
-struct mh_machine
-{
-  strobj_t reg_str;         /* String register */
-  int reg_num;              /* Numeric register */
-
-  strobj_t arg_str;         /* String argument */
-  long arg_num;             /* Numeric argument */
-  
-  size_t pc;                /* Program counter */
-  size_t progsize;          /* Size of allocated program*/
-  mh_instr_t *prog;         /* Program itself */
-  int stop;                 /* Stop execution immediately */
-
-  struct obstack stk;       /* Output buffer */
-  size_t width;             /* Output buffer width */
-  size_t ind;               /* Output buffer index */
-
-  mu_list_t addrlist;          /* The list of email addresses output this far */
-  int fmtflags;             /* Current formatting flags */
-  
-  mu_message_t message;        /* Current message */
-  size_t msgno;             /* Its number */
-};
-
 static char *_get_builtin_name (mh_builtin_fp ptr);
 
 /* Functions for handling string objects. */
@@ -73,13 +40,8 @@ strobj_free (strobj_t *obj)
   obj->ptr = NULL;
 }
 
-#define strobj_ptr(p) ((p)->ptr ? (p)->ptr : "")
-#define strobj_len(p) (strobj_is_null(p) ? 0 : strlen((p)->ptr))
-#define strobj_is_null(p) ((p)->ptr == NULL)
-#define strobj_is_static(p) ((p)->size == 0)
-
-static void
-strobj_create (strobj_t *lvalue, char *str)
+void
+strobj_create (strobj_t *lvalue, const char *str)
 {
   if (!str)
     {
@@ -94,14 +56,14 @@ strobj_create (strobj_t *lvalue, char *str)
     }
 }
 
-static void
+void
 strobj_set (strobj_t *lvalue, char *str)
 {
   lvalue->size = 0;
   lvalue->ptr = str;
 }
 
-static void
+void
 strobj_assign (strobj_t *lvalue, strobj_t *rvalue)
 {
   strobj_free (lvalue);
@@ -110,7 +72,7 @@ strobj_assign (strobj_t *lvalue, strobj_t *rvalue)
   rvalue->ptr = NULL;
 }
 
-static void
+void
 strobj_copy (strobj_t *lvalue, strobj_t *rvalue)
 {
   if (strobj_is_null (rvalue))
@@ -125,7 +87,7 @@ strobj_copy (strobj_t *lvalue, strobj_t *rvalue)
     }
 }
 
-static void
+void
 strobj_realloc (strobj_t *obj, size_t length)
 {
   if (strobj_is_static (obj))
@@ -317,14 +279,6 @@ print_fmt_string (struct mh_machine *mach, size_t fmtwidth, char *str)
     }
 }
 
-/* FIXME: width? */
-static void
-print_obj (struct mh_machine *mach, size_t width, strobj_t *obj)
-{
-  if (!strobj_is_null (obj))
-    print_string (mach, 0, strobj_ptr (obj));
-}
-
 static void
 reset_fmt_defaults (struct mh_machine *mach)
 {
@@ -404,11 +358,10 @@ addr_cmp (void *item, void *data)
   mu_address_get_count (a, &count);
   for (i = 1; rc == 0 && i <= count; i++)
     {
-      char *str;
-      if (mu_address_aget_email (a, i, &str))
+      const char *str;
+      if (mu_address_sget_email (a, i, &str))
 	continue;
       rc = mu_address_contains_email (b, str);
-      free (str);
     }
   return rc;
 }
@@ -601,7 +554,7 @@ mh_format (mh_format_t *fmt, mu_message_t msg, size_t msgno,
 	  break;
 
 	default:
-	  mh_error (_("INTERNAL ERROR: Unknown opcode: %x"), opcode);
+	  mu_error (_("INTERNAL ERROR: Unknown opcode: %x"), opcode);
 	  abort ();
 	}
     }
@@ -807,7 +760,7 @@ mh_format_dump (mh_format_t *fmt)
 	  break;
 
 	default:
-	  mh_error ("Unknown opcode: %x", opcode);
+	  mu_error ("Unknown opcode: %x", opcode);
 	  abort ();
 	}
       printf ("\n");
@@ -831,7 +784,7 @@ mh_format_free (mh_format_t *fmt)
 static void
 builtin_not_implemented (char *name)
 {
-  mh_error ("%s is not yet implemented.", name);
+  mu_error ("%s is not yet implemented.", name);
 }
 
 static void
@@ -944,7 +897,7 @@ builtin_divide (struct mh_machine *mach)
   if (!mach->arg_num)
     {
       /* TRANSLATORS: Do not translate the word 'format'! */
-      mh_error (_("format: divide by zero"));
+      mu_error (_("format: divide by zero"));
       mach->stop = 1;
     }
   else
@@ -956,7 +909,7 @@ builtin_modulo (struct mh_machine *mach)
 {
   if (!mach->arg_num)
     {
-      mh_error (_("format: divide by zero"));
+      mu_error (_("format: divide by zero"));
       mach->stop = 1;
     }
   else
@@ -1091,7 +1044,7 @@ _parse_date (struct mh_machine *mach, struct tm *tm, mu_timezone *tz)
     {
       time_t t;
       
-      /*mh_error ("can't parse date: [%s]", date);*/
+      /*mu_error ("can't parse date: [%s]", date);*/
       time (&t);
       *tm = *localtime (&t);
       tz->utc_offset = mu_utc_offset ();
@@ -1513,8 +1466,7 @@ static void
 builtin_addr (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
+  const char *str;
   int rc;
   
   rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
@@ -1522,8 +1474,8 @@ builtin_addr (struct mh_machine *mach)
   if (rc)
     return;
 
-  if (mu_address_get_email (addr, 1, buf, sizeof buf, &n) == 0)
-    strobj_create (&mach->arg_str, buf);
+  if (mu_address_sget_email (addr, 1, &str) == 0)
+    strobj_create (&mach->arg_str, str);
   mu_address_destroy (&addr);
 }
 
@@ -1532,8 +1484,7 @@ static void
 builtin_pers (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
+  const char *str;
   int rc;
   
   rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
@@ -1541,11 +1492,10 @@ builtin_pers (struct mh_machine *mach)
   if (rc)
     return;
 
-  if (mu_address_get_personal (addr, 1, buf, sizeof buf, &n) == 0
-      && n > 1)
+  if (mu_address_sget_personal (addr, 1, &str) == 0 && str)
     {
       char *p;
-      asprintf (&p, "\"%s\"", buf);
+      asprintf (&p, "\"%s\"", str);
       strobj_create (&mach->arg_str, p);
       free (p);
     }
@@ -1558,8 +1508,7 @@ static void
 builtin_note (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
+  const char *str;
   int rc;
   
   rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
@@ -1567,8 +1516,8 @@ builtin_note (struct mh_machine *mach)
   if (rc)
     return;
 
-  if (mu_address_get_comments (addr, 1, buf, sizeof buf, &n) == 0)
-    strobj_create (&mach->arg_str, buf);
+  if (mu_address_sget_comments (addr, 1, &str) == 0)
+    strobj_create (&mach->arg_str, str);
   mu_address_destroy (&addr);
 }
 
@@ -1577,8 +1526,7 @@ static void
 builtin_mbox (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
+  char *str;
   int rc;
   
   rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
@@ -1586,12 +1534,13 @@ builtin_mbox (struct mh_machine *mach)
   if (rc)
     return;
 
-  if (mu_address_get_email (addr, 1, buf, sizeof buf, &n) == 0)
+  if (mu_address_aget_email (addr, 1, &str) == 0)
     {
-      char *p = strchr (buf, '@');
+      char *p = strchr (str, '@');
       if (p)
 	*p = 0;
       strobj_create (&mach->arg_str, p);
+      free (str);
     }
   mu_address_destroy (&addr);
 }
@@ -1601,15 +1550,14 @@ static void
 builtin_mymbox (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
+  const char *str;
   
   mach->arg_num = 0;
   if (mu_address_create (&addr, strobj_ptr (&mach->arg_str)))
     return;
 
-  if (mu_address_get_email (addr, 1, buf, sizeof buf, &n) == 0)
-    mach->arg_num = mh_is_my_name (buf);
+  if (mu_address_sget_email (addr, 1, &str) == 0 && str)
+    mach->arg_num = mh_is_my_name (str);
   mu_address_destroy (&addr);
 }
 
@@ -1618,8 +1566,7 @@ static void
 builtin_host (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
+  char *str;
   int rc;
   
   rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
@@ -1627,11 +1574,12 @@ builtin_host (struct mh_machine *mach)
   if (rc)
     return;
 
-  if (mu_address_get_email (addr, 1, buf, sizeof buf, &n) == 0)
+  if (mu_address_aget_email (addr, 1, &str) == 0)
     {
-      char *p = strchr (buf, '@');
+      char *p = strchr (str, '@');
       if (p)
 	strobj_create (&mach->arg_str, p+1);
+      free (str);
     }
   mu_address_destroy (&addr);
 }
@@ -1641,17 +1589,15 @@ static void
 builtin_nohost (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
-  int rc;
+  const char *str;
   
-  rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
+  int rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
   strobj_free (&mach->arg_str);
   if (rc)
     return;
 
-  if (mu_address_get_email (addr, 1, buf, sizeof buf, &n) == 0)
-    mach->arg_num = strchr (buf, '@') != NULL;
+  if (mu_address_sget_email (addr, 1, &str) == 0 && str)
+    mach->arg_num = strchr (str, '@') != NULL;
   else
     mach->arg_num = 0;
   mu_address_destroy (&addr);
@@ -1663,20 +1609,19 @@ static void
 builtin_type (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
   int rc;
-
+  const char *str;
+  
   rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
   strobj_free (&mach->arg_str);
   if (rc)
     return;
 
-  if (mu_address_get_email (addr, 1, buf, sizeof buf, &n) == 0)
+  if (mu_address_sget_email (addr, 1, &str) == 0 && str)
     {
-      if (strchr (buf, '@'))
+      if (strchr (str, '@'))
 	mach->arg_num = 1;
-      else if (strchr (buf, '@'))
+      else if (strchr (str, '!'))
 	mach->arg_num = -1;
       else
 	mach->arg_num = 0; /* assume local */
@@ -1691,16 +1636,13 @@ static void
 builtin_path (struct mh_machine *mach)
 {
   mu_address_t addr;
-  size_t n;
-  char buf[80];
-  int rc;
-  
-  rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
+  const char *str;
+  int rc = mu_address_create (&addr, strobj_ptr (&mach->arg_str));
   strobj_free (&mach->arg_str);
   if (rc)
     return;
-  if (mu_address_get_route (addr, 1, buf, sizeof buf, &n))
-    strobj_create (&mach->arg_str, buf);
+  if (mu_address_sget_route (addr, 1, &str))
+    strobj_create (&mach->arg_str, str);
   mu_address_destroy (&addr);
 }
 
@@ -1729,7 +1671,7 @@ builtin_formataddr (struct mh_machine *mach)
   size_t size;
   int i;
   size_t num;
-  char *buf;
+  const char *buf;
   
   if (strobj_len (&mach->reg_str) == 0)
     dest = NULL;
@@ -1745,7 +1687,7 @@ builtin_formataddr (struct mh_machine *mach)
   mu_address_get_count (addr, &num);
   for (i = 1; i <= num; i++)
     {
-      if (mu_address_aget_email (addr, i, &buf) == 0)
+      if (mu_address_sget_email (addr, i, &buf) == 0)
 	{
 	  if ((rcpt_mask & RCPT_ME) || !mh_is_my_name (buf))
 	    {
@@ -1759,7 +1701,6 @@ builtin_formataddr (struct mh_machine *mach)
 	      else
 		mu_address_destroy (&subaddr);
 	    }
-	  free (buf);
 	}
     }
 
@@ -1873,7 +1814,7 @@ builtin_rcpt (struct mh_machine *mach)
   int rc = mh_decode_rcpt_flag (strobj_ptr (&mach->arg_str));
   if (rc == RCPT_NONE)
     {
-      mh_error (_("Invalid recipient mask"));
+      mu_error (_("invalid recipient mask"));
       /* try to continue anyway */
     }
   mach->arg_num = rc & rcpt_mask;
