@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005, 2006, 2007, 2008,
+   2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,15 +38,15 @@ static char args_doc[] = N_("[+folder]");
 /* GNU options */
 static struct argp_option options[] = {
   {"folder",  ARG_FOLDER, N_("FOLDER"), 0,
-   N_("Specify the folder to delete")},
+   N_("specify the folder to delete")},
   {"interactive", ARG_INTERACTIVE, N_("BOOL"), OPTION_ARG_OPTIONAL,
-    N_("Interactive mode: ask for confirmation before removing each folder")},
+    N_("interactive mode: ask for confirmation before removing each folder")},
   {"nointeractive", ARG_NOINTERACTIVE, NULL, OPTION_HIDDEN, ""},
   {"recursive", ARG_RECURSIVE, NULL, 0,
-   N_("Recursively delete all subfolders")},
+   N_("recursively delete all subfolders")},
   {"norecursive", ARG_NORECURSIVE, NULL, OPTION_HIDDEN, ""},
   {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
   { 0 }
 };
 
@@ -123,7 +124,7 @@ rmf (const char *name)
 
   if (!dir)
     {
-      mu_error (_("Cannot scan folder %s: %s"), name, strerror (errno));
+      mu_error (_("cannot scan folder %s: %s"), name, strerror (errno));
       return 1;
     }
 
@@ -142,7 +143,7 @@ rmf (const char *name)
       asprintf (&p, "%s/%s", name, entry->d_name);
       if (stat (p, &st) < 0)
 	{
-	  mu_error (_("Cannot stat %s: %s"), p, strerror (errno));
+	  mu_diag_funcall (MU_DIAG_ERROR, "stat", p, errno);
 	}
       else if (S_ISDIR (st.st_mode))
 	{
@@ -159,7 +160,7 @@ rmf (const char *name)
 	{
 	  if (unlink (p))
 	    {
-	      mu_error (_("Cannot unlink %s: %s"), p, strerror (errno));
+	      mu_diag_funcall (MU_DIAG_ERROR, "unlink", p, errno);
 	      failures++;
 	    }
 	}
