@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 
-   2005, 2007, 2008 Free Software Foundation, Inc.
+   2005, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,41 +67,41 @@ static char doc[] = N_("GNU pop3d -- the POP3 daemon");
 
 static struct argp_option options[] = {
 #define GRP 0
-  { "foreground", OPT_FOREGROUND, 0, 0, N_("Remain in foreground."), GRP+1},
-  { "inetd",  'i', 0, 0, N_("Run in inetd mode"), GRP+1},
+  { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), GRP+1},
+  { "inetd",  'i', 0, 0, N_("run in inetd mode"), GRP+1},
   { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
-    N_("Runs in daemon mode with a maximum of NUMBER children"), GRP+1 },
+    N_("runs in daemon mode with a maximum of NUMBER children"), GRP+1 },
 #undef GRP
 
 #define GRP 5
   {"undelete", 'u', NULL, OPTION_HIDDEN,
-   N_("Undelete all messages on startup"), GRP+1},
+   N_("undelete all messages on startup"), GRP+1},
   {"expire", OPT_EXPIRE, N_("DAYS"), OPTION_HIDDEN,
-   N_("Expire read messages after the given number of days"), GRP+1},
+   N_("expire read messages after the given number of days"), GRP+1},
   {"delete-expired", OPT_EXPIRE_ON_EXIT, NULL, OPTION_HIDDEN,
-   N_("Delete expired messages upon closing the mailbox"), GRP+1},
+   N_("delete expired messages upon closing the mailbox"), GRP+1},
 #ifdef WITH_TLS
   {"tls-required", OPT_TLS_REQUIRED, NULL, OPTION_HIDDEN,
-   N_("Always require STLS before entering authentication phase")},
+   N_("always require STLS before entering authentication phase")},
 #endif
 #undef GRP
   
 #define GRP 10
 #ifdef ENABLE_LOGIN_DELAY
   {"login-delay", OPT_LOGIN_DELAY, N_("SECONDS"), OPTION_HIDDEN,
-   N_("Allowed delay between two successive logins"), GRP+1},
+   N_("allowed delay between two successive logins"), GRP+1},
   {"stat-file", OPT_STAT_FILE, N_("FILENAME"), OPTION_HIDDEN,
-   N_("Name of login statistics file"), GRP+1},
+   N_("name of login statistics file"), GRP+1},
 #endif
   
 #undef GRP
 
 #define GRP 20
   { "bulletin-source", OPT_BULLETIN_SOURCE, N_("MBOX"), OPTION_HIDDEN,
-    N_("Set source mailbox to get bulletins from"), GRP+1 },
+    N_("set source mailbox to get bulletins from"), GRP+1 },
 #ifdef USE_DBM
   { "bulletin-db", OPT_BULLETIN_DB, N_("FILE"), OPTION_HIDDEN,
-    N_("Set the bulletin database file name"), GRP+1 },
+    N_("set the bulletin database file name"), GRP+1 },
 #endif
 #undef GRP
   
@@ -117,6 +117,7 @@ cb_bulletin_source (mu_debug_t debug, void *data, mu_config_value_t *val)
   return 0;
 }
 
+#ifdef USE_DBM
 static int
 cb_bulletin_db (mu_debug_t debug, void *data, mu_config_value_t *val)
 {
@@ -125,6 +126,7 @@ cb_bulletin_db (mu_debug_t debug, void *data, mu_config_value_t *val)
   set_bulletin_db (val->v.string); /* FIXME: Error reporting? */
   return 0;
 }
+#endif
 
 static struct mu_cfg_param pop3d_cfg_param[] = {
   { "undelete", mu_cfg_bool, &undelete_on_startup, 0, NULL,
@@ -254,12 +256,12 @@ pop3d_parse_opt (int key, char *arg, struct argp_state *astate)
 int
 pop3d_get_client_address (int fd, struct sockaddr_in *pcs)
 {
-  mu_diag_output (MU_DIAG_INFO, _("Incoming connection opened"));
+  mu_diag_output (MU_DIAG_INFO, _("incoming connection opened"));
 
   /* log information on the connecting client. */
   if (debug_mode)
     {
-      mu_diag_output (MU_DIAG_INFO, _("Started in debugging mode"));
+      mu_diag_output (MU_DIAG_INFO, _("started in debugging mode"));
       return 1;
     }
   else
@@ -268,7 +270,7 @@ pop3d_get_client_address (int fd, struct sockaddr_in *pcs)
       if (getpeername (fd, (struct sockaddr*) pcs, &len) < 0)
 	{
 	  mu_diag_output (MU_DIAG_ERROR,
-			  _("Cannot obtain IP address of client: %s"),
+			  _("cannot obtain IP address of client: %s"),
 			  strerror (errno));
 	  return 1;
 	}
@@ -299,9 +301,7 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
   /* Prepare the shared secret for APOP.  */
   {
     char *local_hostname;
-    local_hostname = malloc (MAXHOSTNAMELEN + 1);
-    if (local_hostname == NULL)
-      pop3d_abquit (ERR_NO_MEM);
+    local_hostname = mu_alloc (MAXHOSTNAMELEN + 1);
 
     /* Get our canonical hostname. */
     {
@@ -315,9 +315,7 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
 	}
     }
 
-    md5shared = malloc (strlen (local_hostname) + 51);
-    if (md5shared == NULL)
-      pop3d_abquit (ERR_NO_MEM);
+    md5shared = mu_alloc (strlen (local_hostname) + 51);
 
     snprintf (md5shared, strlen (local_hostname) + 50, "<%u.%u@%s>", getpid (),
 	      (unsigned)time (NULL), local_hostname);
@@ -327,15 +325,16 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
   /* Lets boogie.  */
   pop3d_outf ("+OK POP3 Ready %s\r\n", md5shared);
 
-  while (state != UPDATE)
+  while (state != UPDATE && state != ABORT)
     {
-      char *buf, *arg, *cmd;
-
+      char *buf;
+      char *arg, *cmd;
+      pop3d_command_handler_t handler;
+      
       pop3d_flush_output ();
       status = OK;
       buf = pop3d_readline (buffer, sizeof (buffer));
-      cmd = pop3d_cmd (buf);
-      arg = pop3d_args (buf);
+      pop3d_parse_command (buf, &cmd, &arg);
 
       /* The mailbox size needs to be check to make sure that we are in
 	 sync.  Some other applications may not respect the *.lock or
@@ -361,79 +360,13 @@ pop3d_mainloop (int fd, FILE *infile, FILE *outfile)
 	status = ERR_TOO_LONG;
       else if (strlen (cmd) > 4)
 	status = ERR_BAD_CMD;
-      else if (strncasecmp (cmd, "RETR", 4) == 0)
-	status = pop3d_retr (arg);
-      else if (strncasecmp (cmd, "DELE", 4) == 0)
-	status = pop3d_dele (arg);
-      else if (strncasecmp (cmd, "USER", 4) == 0)
-	status = pop3d_user (arg);
-      else if (strncasecmp (cmd, "QUIT", 4) == 0)
-	status = pop3d_quit (arg);
-      else if (strncasecmp (cmd, "APOP", 4) == 0)
-	status = pop3d_apop (arg);
-      else if (strncasecmp (cmd, "AUTH", 4) == 0)
-	status = pop3d_auth (arg);
-      else if (strncasecmp (cmd, "STAT", 4) == 0)
-	status = pop3d_stat (arg);
-      else if (strncasecmp (cmd, "LIST", 4) == 0)
-	status = pop3d_list (arg);
-      else if (strncasecmp (cmd, "NOOP", 4) == 0)
-	status = pop3d_noop (arg);
-      else if (strncasecmp (cmd, "RSET", 4) == 0)
-	status = pop3d_rset (arg);
-      else if ((strncasecmp (cmd, "TOP", 3) == 0) && (strlen (cmd) == 3))
-	status = pop3d_top (arg);
-      else if (strncasecmp (cmd, "UIDL", 4) == 0)
-	status = pop3d_uidl (arg);
-      else if (strncasecmp (cmd, "CAPA", 4) == 0)
-	status = pop3d_capa (arg);
-#ifdef WITH_TLS
-      else if ((strncasecmp (cmd, "STLS", 4) == 0) && tls_available)
-	{
-	  status = pop3d_stls (arg);
-	  if (status)
-	    {
-	      mu_diag_output (MU_DIAG_ERROR, _("Session terminated"));
-	      break;
-	    }
-	}
-#endif /* WITH_TLS */
+      else if ((handler = pop3d_find_command (cmd)) != NULL)
+	status = handler (arg);
       else
 	status = ERR_BAD_CMD;
 
-      if (status == OK)
-	; /* Everything is good.  */
-      else if (status == ERR_WRONG_STATE)
-	pop3d_outf ("-ERR " BAD_STATE "\r\n");
-      else if (status == ERR_BAD_ARGS)
-	pop3d_outf ("-ERR " BAD_ARGS "\r\n");
-      else if (status == ERR_NO_MESG)
-	pop3d_outf ("-ERR " NO_MESG "\r\n");
-      else if (status == ERR_MESG_DELE)
-	pop3d_outf ("-ERR " MESG_DELE "\r\n");
-      else if (status == ERR_NOT_IMPL)
-	pop3d_outf ("-ERR " NOT_IMPL "\r\n");
-      else if (status == ERR_BAD_CMD)
-	pop3d_outf ("-ERR " BAD_COMMAND "\r\n");
-      else if (status == ERR_BAD_LOGIN)
-	pop3d_outf ("-ERR " BAD_LOGIN "\r\n");
-      else if (status == ERR_MBOX_LOCK)
-	pop3d_outf ("-ERR [IN-USE] " MBOX_LOCK "\r\n");
-      else if (status == ERR_TOO_LONG)
-	pop3d_outf ("-ERR " TOO_LONG "\r\n");
-      else if (status == ERR_FILE)
-	pop3d_outf ("-ERR " FILE_EXP "\r\n");
-#ifdef WITH_TLS
-      else if (status == ERR_TLS_ACTIVE)
-	pop3d_outf ("-ERR " TLS_ACTIVE "\r\n");
-#endif /* WITH_TLS */
-      else if (status == ERR_LOGIN_DELAY)
-	pop3d_outf ("-ERR [LOGIN-DELAY] " LOGIN_DELAY "\r\n");
-      else
-	pop3d_outf ("-ERR unknown error\r\n");
-
-      free (cmd);
-      free (arg);
+      if (status != OK)
+	pop3d_outf ("-ERR %s\r\n", pop3d_error_string (status));
     }
 
   pop3d_bye ();
@@ -451,6 +384,12 @@ pop3d_connection (int fd, struct sockaddr *sa, int salen, void *data,
   return 0;
 }
 
+static void
+pop3d_alloc_die ()
+{
+  pop3d_abquit (ERR_NO_MEM);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -484,6 +423,8 @@ main (int argc, char **argv)
   mu_m_server_set_default_port (server, 110);
   mu_m_server_set_timeout (server, 600);
   mu_m_server_set_strexit (server, mu_strexit);
+
+  mu_alloc_die_hook = pop3d_alloc_die;
   
   if (mu_app_init (&argp, pop3d_argp_capa, pop3d_cfg_param, 
 		   argc, argv, 0, NULL, server))
@@ -513,20 +454,19 @@ main (int argc, char **argv)
 	{
 	  if (errno == 0 || errno == ENOENT)
             {
-               mu_error (_("%s: No such group"), "mail");
+               mu_error (_("%s: no such group"), "mail");
                exit (EX_CONFIG);
             }
           else
             {
-	       mu_error (_("Error getting mail group: %s"), 
-                         mu_strerror (errno));
-	       exit (EX_OSERR);
+	      mu_diag_funcall (MU_DIAG_ERROR, "getgrnam", "mail", errno);
+	      exit (EX_OSERR);
             }
 	}
 
       if (setgid (gr->gr_gid) == -1)
 	{
-	  mu_error (_("Error setting mail group: %s"), mu_strerror (errno));
+	  mu_error (_("error setting mail group: %s"), mu_strerror (errno));
 	  exit (EX_OSERR);
 	}
     }
@@ -553,7 +493,11 @@ main (int argc, char **argv)
 #ifdef WITH_TLS
   tls_available = mu_check_tls_environment ();
   if (tls_available)
-    tls_available = mu_init_tls_libs ();
+    {
+      tls_available = mu_init_tls_libs ();
+      if (tls_available)
+	enable_stls ();
+    }
 #endif /* WITH_TLS */
 
   /* Actually run the daemon.  */
@@ -572,7 +516,7 @@ main (int argc, char **argv)
     }
   
   if (status)
-    mu_error (_("Main loop status: %s"), mu_strerror (status));	  
+    mu_error (_("main loop status: %s"), mu_strerror (status));	  
   /* Close the syslog connection and exit.  */
   closelog ();
   return status ? EX_SOFTWARE : EX_OK;
