@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007,
-   2008 Free Software Foundation, Inc.
+   2008, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -68,11 +68,11 @@ void (*argp_program_version_hook) (FILE *stream, struct argp_state *state) =
 static struct argp_option options[] = 
 {
   { NULL, 0, NULL, 0, N_("Actions are:"), 1 },
-  { "add", 'a', 0, 0, N_("Add user"), 1 },
-  { "modify", 'm', 0, 0, N_("Modify user's record (change password)"), 1 },
-  { "delete", 'd', 0, 0, N_("Delete user's record"), 1 },
-  { "list", 'l', 0, 0, N_("List the contents of DBM file"), 1 },
-  { "create", 'c', 0, 0, N_("Create the DBM from a plaintext file"), 1 },
+  { "add", 'a', 0, 0, N_("add user"), 1 },
+  { "modify", 'm', 0, 0, N_("modify user's record (change password)"), 1 },
+  { "delete", 'd', 0, 0, N_("delete user's record"), 1 },
+  { "list", 'l', 0, 0, N_("list the contents of DBM file"), 1 },
+  { "create", 'c', 0, 0, N_("create the DBM from a plaintext file"), 1 },
 
   { NULL, 0, NULL, 0,
     N_("Default action is:\n"
@@ -80,11 +80,11 @@ static struct argp_option options[] =
     "  For a user: --modify --username <username>\n"), 2 },
   
   { NULL, 0, NULL, 0, N_("Options are:"), 3 },
-  { "file", 'f', N_("FILE"), 0, N_("Read input from FILE (default stdin)"), 3 },
-  { "output", 'o', N_("FILE"), 0, N_("Direct output to file"), 3 },
-  { "password", 'p', N_("STRING"), 0, N_("Specify user's password"), 3 },
-  { "user", 'u', N_("USERNAME"), 0, N_("Specify user name"), 3 },
-  { "permissions", 'P', N_("PERM"), 0, N_("Force given permissions on the database"), 3 },
+  { "file", 'f', N_("FILE"), 0, N_("read input from FILE (default stdin)"), 3 },
+  { "output", 'o', N_("FILE"), 0, N_("direct output to file"), 3 },
+  { "password", 'p', N_("STRING"), 0, N_("specify user's password"), 3 },
+  { "user", 'u', N_("USERNAME"), 0, N_("specify user name"), 3 },
+  { "permissions", 'P', N_("PERM"), 0, N_("force given permissions on the database"), 3 },
   { NULL, }
 };
 
@@ -108,13 +108,13 @@ set_db_perms (struct argp_state *astate, char *opt, int *pperm)
 {
   int perm = 0;
    
-  if (isdigit(opt[0]))
+  if (mu_isdigit(opt[0]))
     {
       char *p;
       perm = strtoul (opt, &p, 8);
       if (*p)
 	{
-	  argp_error (astate, _("Invalid octal number: %s"), opt);
+	  argp_error (astate, _("invalid octal number: %s"), opt);
 	  exit (EX_USAGE);
 	}
     }
@@ -237,8 +237,8 @@ check_user_perm (int action, struct action_data *ap)
 	  DBM_FILE db;
 	  if (mu_dbm_open (ap->input_name, &db, MU_STREAM_CREAT, permissions))
 	    {
-	      mu_error (_("Cannot create %s: %s"),
-			ap->input_name, mu_strerror (errno));
+	      mu_diag_funcall (MU_DIAG_ERROR, "mu_dbm_open",
+			       ap->input_name, errno);
 	      exit (EX_SOFTWARE);
 	    }
 	  mu_dbm_close (db);
@@ -246,8 +246,7 @@ check_user_perm (int action, struct action_data *ap)
 	}
       else
 	{
-	  mu_error (_("Cannot stat %s: %s"), ap->input_name, 
-                    mu_strerror (errno));
+	  mu_diag_funcall (MU_DIAG_ERROR, "stat", ap->input_name, errno);
 	  exit (EX_OSERR);
 	}
     }
@@ -285,7 +284,8 @@ action_list (struct action_data *ap)
   check_user_perm (ACT_LIST, ap);
   if (mu_dbm_open (ap->input_name, &db, MU_STREAM_READ, 0))
     {
-      mu_error (_("Cannot open %s: %s"), ap->input_name, mu_strerror (errno));
+      mu_error (_("cannot open file %s: %s"),
+		ap->input_name, mu_strerror (errno));
       return 1;
     }
   
@@ -294,7 +294,7 @@ action_list (struct action_data *ap)
       fp = fopen (ap->output_name, "w");
       if (!fp)
 	{
-	  mu_error (_("Cannot create %s: %s"), ap->output_name, mu_strerror (errno));
+	  mu_error (_("cannot create file %s: %s"), ap->output_name, mu_strerror (errno));
 	  return 1;
 	}
     }
@@ -309,7 +309,7 @@ action_list (struct action_data *ap)
       MU_DATUM_SIZE (key) = strlen (ap->username);
       if (mu_dbm_fetch (db, key, &contents))
 	{
-	  mu_error (_("No such user: %s"), ap->username);
+	  mu_error (_("no such user: %s"), ap->username);
 	}
       else
 	{
@@ -360,7 +360,8 @@ action_create (struct action_data *ap)
       fp = fopen (ap->input_name, "r");
       if (!fp)
 	{
-	  mu_error (_("Cannot open %s: %s"), ap->input_name, mu_strerror (errno));
+	  mu_error (_("cannot open file %s: %s"),
+		    ap->input_name, mu_strerror (errno));
 	  return 1;
 	}
     }
@@ -374,7 +375,7 @@ action_create (struct action_data *ap)
     ap->output_name = APOP_PASSFILE;
   if (mu_dbm_open (ap->output_name, &db, MU_STREAM_CREAT, permissions))
     {
-      mu_error (_("Cannot create %s: %s"), ap->output_name, mu_strerror (errno));
+      mu_error (_("cannot create database %s: %s"), ap->output_name, mu_strerror (errno));
       return 1;
     }
 
@@ -434,7 +435,8 @@ open_io (int action, struct action_data *ap, DBM_FILE *db, int *not_owner)
     *not_owner = rc;
   if (mu_dbm_open (ap->input_name, db, MU_STREAM_RDWR, permissions))
     {
-      mu_error (_("Cannot open %s: %s"), ap->input_name, mu_strerror (errno));
+      mu_error (_("cannot open file %s: %s"),
+		ap->input_name, mu_strerror (errno));
       return 1;
     }
   return 0;
@@ -474,7 +476,7 @@ action_add (struct action_data *ap)
   
   if (!ap->username)
     {
-      mu_error (_("Missing username to add"));
+      mu_error (_("missing username to add"));
       return 1;
     }
 
@@ -492,7 +494,7 @@ action_add (struct action_data *ap)
 
   rc = mu_dbm_insert (db, key, contents, 1);
   if (rc)
-    mu_error (_("Cannot store datum"));
+    mu_error (_("cannot store datum"));
 
   mu_dbm_close (db);
   return rc;
@@ -507,7 +509,7 @@ action_delete (struct action_data *ap)
   
   if (!ap->username)
     {
-      mu_error (_("Missing username to delete"));
+      mu_error (_("missing username to delete"));
       return 1;
     }
 
@@ -519,7 +521,7 @@ action_delete (struct action_data *ap)
 
   rc = mu_dbm_delete (db, key);
   if (rc)
-    mu_error (_("Cannot remove record for %s"), ap->username);
+    mu_error (_("cannot remove record for %s"), ap->username);
 
   mu_dbm_close (db);
   return rc;
@@ -539,7 +541,7 @@ action_chpass (struct action_data *ap)
 
   if (!ap->username)
     {
-      mu_error (_("Missing username"));
+      mu_error (_("missing username"));
       return 1;
     }
 
@@ -550,7 +552,7 @@ action_chpass (struct action_data *ap)
   MU_DATUM_SIZE (key) = strlen (ap->username);
   if (mu_dbm_fetch (db, key, &contents))
     {
-      mu_error (_("No such user: %s"), ap->username);
+      mu_error (_("no such user: %s"), ap->username);
       return 1;
     }
 
@@ -578,7 +580,7 @@ action_chpass (struct action_data *ap)
   MU_DATUM_SIZE (contents) = strlen (ap->passwd);
   rc = mu_dbm_insert (db, key, contents, 1);
   if (rc)
-    mu_error (_("Cannot replace datum"));
+    mu_error (_("cannot replace datum"));
 
   mu_dbm_close (db);
   return rc;
@@ -592,9 +594,11 @@ popauth_version (FILE *stream, struct argp_state *state)
 #elif defined(WITH_BDB)
 # define FORMAT "Berkeley DB"
 #elif defined(WITH_NDBM)
-# define FORMAT "NDBM"  
+# define FORMAT "NDBM"
 #elif defined(WITH_OLD_DBM)
-# define FORMAT "Old DBM"  
+# define FORMAT "Old DBM"
+#elif defined(WITH_TOKYOCABINET)
+# define FORMAT "Tokyo Cabinet"
 #endif
   printf ("%s\n", argp_program_version);
   printf (_("Database format: %s\n"), FORMAT);
